<?php 
/**
 * @package    Proxim
 * @author     Davison Pro <davis@davisonpro.dev | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.5.0
 * @since      File available since Release 1.0.0
 */

namespace Proxim\Module;

use Db;
use PrePaper;
use PrePaperFile;
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Currency;
use Proxim\Database\DbQuery;
use Proxim\Hook;
use Proxim\Mail;
use Proxim\MailTemplate;
use Proxim\Order\AdditionalService;
use Proxim\Order\File;
use Proxim\Order\Order;
use Proxim\Order\OrderMessage;
use Proxim\Payment;
use Proxim\Preference\Deadline;
use Proxim\Presenter\Order\OrderPresenter;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\User\Customer;
use Proxim\User\Wallet as CustomerWallet;
use Proxim\User\Employee;
use Proxim\User\Referral;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;
use Proxim\Validate;
use Wallet;

abstract class PaymentModule extends Module
{
    public $currencies = true;

    const DEBUG_MODE = false;

    /**
     * List all installed and active payment modules.
     *
     * @see Module::getPaymentModules() if you need a list of module
     *
     * @return array module informations
     */
    public static function getInstalledPaymentModules()
    {
        $hook_payment = 'Payment';
        if (Db::getInstance()->getValue('SELECT `hook_id` FROM ' .  Db::prefix('hook') . 's WHERE `name` = \'paymentOptions\'')) {
            $hook_payment = 'paymentOptions';
        }

        return Db::getInstance()->executeS('
            SELECT DISTINCT m.`module_id`, h.`hook_id`, m.`name`, hm.`position`
            FROM ' .  Db::prefix('module') . ' m
            LEFT JOIN ' .  Db::prefix('hook_module') . ' hm ON hm.`module_id` = m.`module_id`
            LEFT JOIN ' .  Db::prefix('hook') . ' h ON hm.`hook_id` = h.`hook_id`
            WHERE h.`name` = \'' . pSQL($hook_payment) . '\'
        ');
    }

    public static function processPayment($nodeId, $handle, $payload = array())
    {
        global $globals;
        $app = Application::getInstance();
        $smarty = $app->container->smarty;

        Hook::exec('processPayment', $payload);

        switch( $handle ) { 
            case 'order':
                $transaction_id = ArrayUtils::get($payload, 'paymentId');

                $order = new Order( (int) $nodeId );
                if (!Validate::isLoadedObject($order)) {
                    return false;
                } 

                $currency = $order->currency;

                $options['currency'] = array(
                    'symbol' => $currency->symbol,
                    'rate' => $currency->conversion_rate
                );

                $payment_method = ArrayUtils::get($payload, 'payment_method');
                $tax = (float) ArrayUtils::get($payload, 'tax', 0);

                $order->tax = $tax;
                $order->total += $tax;

                $order->payment_method = $payment_method;
                $order->total_paid = $order->total_paid + $order->total;
                $old_status = $order->status_id;

                $payment = new Payment();
                $payment->transaction_id = $transaction_id;
                $payment->currency_id = $order->currency_id;
                $payment->payment_method = $payment_method;
                $payment->conversion_rate = $order->currency_to_usd_rate;
                $payment->node_id = $order->id;
                $payment->node_type = 'order';
                $payment->amount = $order->total;
                $payment->add();

                $preferredWriter = false;
                if ( 
                    $old_status == Order::FAKE || 
                    $old_status == Order::FREE_INQUIRY || 
                    $old_status == Order::INQUIRY
                ) {
                    $customer_deadline = time() + ($order->hrs_customer*60*60);
                    $writer_deadline = time() + ($order->hrs_writer*60*60);
                    $order->customer_deadline = DateUtils::date($customer_deadline);
                    $order->writer_deadline = DateUtils::date($writer_deadline);
 
                    if ($order->progressive_delivery) {
                        $order->status_id = Order::NEW_PAID;
                    } else {

                        if(Configuration::get('NEW_ORDER_STATUS', PROX_ADMIN_SITE_ID)) {
                            $order->status_id = (int) Configuration::get('NEW_ORDER_STATUS', PROX_ADMIN_SITE_ID);
                        } else {
                            $order->status_id = Order::ORDER_PUBLISHED;
                        }
                    }

                    $order->is_paid = 1;
                    $order->paid_at = DateUtils::now();

                    if ($order->requested_writer_id && Configuration::get('AUTOASSIGN_REQUESTED_WRITER', PROX_ADMIN_SITE_ID)) {
                        $writer = new Employee( (int) $order->requested_writer_id );
                        if(Validate::isLoadedObject($writer)) {
                            $preferredWriter = true;

                            $order->status_id = Order::WRITER_ASSIGNED;
                            $order->writer_id = $writer->id;
                            $order->writer_assigned = 1;
                            $order->writer_assigned_at = DateUtils::now();
                            $order->is_writer_confirmed = 0;
                        }
                    }
                }

                $order->update();

                Hook::exec('actionAfterOrderPayment', [
                    'order' => $order,
                ]);

                if ( 
                    $old_status == Order::FAKE || 
                    $old_status == Order::FREE_INQUIRY || 
                    $old_status == Order::INQUIRY
                ) {
                    $customer = new Customer( (int) $order->customer_id );
                    if(Validate::isLoadedObject($customer)) {
                        $customer->total_spent = $customer->total_spent + $order->total;
                        $customer->total_orders = $customer->total_orders+1;

                        /** process affiliates */
                        if ( $customer->referrer_id && !$customer->affiliate_used ) {
                            $referralExists = Referral::referralExists($customer->referrer_id, $customer->id);
                            if(!$referralExists) {
                                $referral = new Referral();
                                $referral->referrer_id = $customer->referrer_id;
                                $referral->referree_id = $customer->id;
                                $referral->code = Tools::randomGen(8, 'NO_NUMERIC');
                                $referral->amount = 10;
                                $referral->referree_used = 1;
                                $referral->add();
                            }

                            $customer->affiliate_used = 1;

                            Hook::exec('actionProcessAffiliate', [
                                'referee_id' => $customer->id,
                                'referrer_id' => $customer->referrer_id
                            ]);
                        }

                        $customer->update();

                        $site = new Site( (int) $customer->site_id );
                        if( Validate::isLoadedObject($site) ) {
                            $site->orders = $site->orders+1;       
                            $site->payments = $site->payments + $order->total;
                            $site->update();
                        }

                        try {
                            $serviceType = ArrayUtils::get($globals['serviceTypes'], (int) $order->service_type);

                            $mailParams = array(
                                'type' => 'order',
                                'order_id' => $order->id,
                                'title' => $order->title,
                                'pages' => $order->pages,
                                'words' => $order->words,
                                'serviceTypeId' => $order->service_type,
                                'serviceTypeTitle' => ArrayUtils::get($serviceType, 'title'),
                                'paperTypeTitle' => $order->paper_type_option,
                                'disciplineTitle' => $order->topic_category_option,
                                'contentTypeTitle' => $order->paper_type_option,
                                'payment_method' => Tools::ucfirst($order->payment_method),
                                'paid_at' => DateUtils::date( strtotime($order->paid_at), null, 'M jS, Y \\a\\t g:i A'),
                                'paid_amount' => formatPrice($order->total, $options),
                                'order_url' => $app->base_uri . '/dashboard/order/' . (int) $order->id,
                                'username' => $customer->name
                            );

                            if(!$customer->is_anonymous) {
                                Mail::Send( 
                                    MailTemplate::TEMPLATE_ORDER_PAYMENT,
                                    'Payment confirmation for order #' . $order->id,
                                    $mailParams,
                                    $customer->email
                                );
                            }
                            
                        } catch(\Exception $e) {
                        }
                    }

                    $receivers = $app->getNotificationEmails( true );

                    $site_url = Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID);

                    $mailParams = array(
                        'type' => 'order',
                        'order_id' => $order->id,
                        'title' => $order->title,
                        'pages' => $order->pages,
                        'words' => $order->words,
                        'serviceTypeId' => $order->service_type,
                        'serviceTypeTitle' => ArrayUtils::get($serviceType, 'title'),
                        'paperTypeTitle' => $order->paper_type_option,
                        'disciplineTitle' => $order->topic_category_option,
                        'contentTypeTitle' => $order->paper_type_option,
                        'admin_site_name' => Configuration::get('SITE_NAME', PROX_ADMIN_SITE_ID),
                        'order_url' => Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID) . '/order/' . (int) $order->id,
                        'admin_order_url' => $site_url . '/order/' . (int) $order->id,
                    ); 

                    foreach ($receivers as $receiver) {
                        try {
                            Mail::Send(
                                MailTemplate::TEMPLATE_ORDER_NEW_AVAILABLE,
                                'New Order #' . $order->id . ' available',
                                $mailParams,
                                $receiver,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                PROX_ADMIN_SITE_ID
                            );
                        } catch(\Exception $e) {
                        }
                    }

                    if($preferredWriter) {
                        try {
                            $writer = new Employee( (int) $order->requested_writer_id );
                            $smarty->assign([
                                'order_id' => $order->id,
                                'username' => $writer->first_name . ' ' . $writer->last_name
                            ]);
                                
                            $confirm_order_body = $smarty->fetch('messages/confirm_order.tpl');
                        
                            $message = new OrderMessage();
                            $message->order_id = $order->id;
                            $message->sender_id = Employee::START_ID;
                            $message->from_department = DEPARTMENT_SUPPORT;
                            $message->to_department = DEPARTMENT_WRITER;
                            $message->subject = "Order #".$order->id.": Please confirm that you are working";
                            $message->body = $confirm_order_body;
                            $message->is_verified = 1;
                            $message->verified_at = DateUtils::now();
                            $message->add();
                        
                            Mail::Send(
                                MailTemplate::TEMPLATE_ORDER_ASSIGN,
                                $message->subject, 
                                array(
                                    'order_id' => $order->id,
                                    'order_title' => $order->title,
                                    'order_url' => $app->base_uri . '/order/' . $order->id,
                                    'username' => $writer->first_name . ' ' . $writer->last_name,
                                    'message' => Tools::nl2br($confirm_order_body)
                                ),
                                $writer->email,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                PROX_ADMIN_SITE_ID
                            );
                        } catch( \Exception $e ) {
                        }
                    }
                }

                return true;
                break;

            case 'add-payment':
                $transaction_id =ArrayUtils::get($payload, 'paymentId');
                $addPayment = new AdditionalService( (int) $nodeId );
                if (!Validate::isLoadedObject($addPayment)) {
                    return false;
                } 
    
                $order = new Order( (int) $addPayment->order_id );
                if (!Validate::isLoadedObject($order)) {
                    return false;
                }

                if (!$addPayment->is_paid) {
                    // pages
                    if($addPayment->pages) $order->pages = $order->pages + $addPayment->pages;
    
                    // charts
                    if($addPayment->charts) $order->charts = $addPayment->charts + $addPayment->charts;
    
                    // slides
                    if($addPayment->slides) $order->slides = $addPayment->slides + $addPayment->slides;
    
                    // writer samples
                    if($addPayment->writer_samples) $order->writer_samples = 1;
    
                    // used sources
                    if($addPayment->used_sources) $order->used_sources = 1;
    
                    // expert proofreading
                    if($addPayment->expert_proofreading) $order->expert_proofreading = 1;
    
                    // progressive delivery
                    if($addPayment->progressive_delivery) $order->progressive_delivery = 1;
    
                    // writer category 
                    if($addPayment->writer_category_id) $order->writer_category_id = $addPayment->writer_category_id;
    
                    // shorten deadline
                    if($addPayment->tariff_id) {
                        $tariff = new Deadline($addPayment->tariff_id);
                        if ( Validate::isLoadedObject($tariff) ) {
                            $price_per_page = (float) $tariff->price_per_page;
                            $hrs_customer = (int) $tariff->hrs_customer;
                            $hrs_writer = (int) $tariff->hrs_writer;
                            $customer_deadline = time() + ($hrs_customer*60*60);
                            $writer_deadline = time() + ($hrs_writer*60*60);
    
                            $order->tariff_id = $addPayment->tariff_id;
                            $order->price_per_page = $price_per_page;
                            $order->hrs_customer = $hrs_customer;
                            $order->hrs_writer = $hrs_writer;
                            $order->customer_deadline = DateUtils::date($customer_deadline);
                            $order->writer_deadline = DateUtils::date($writer_deadline);
                        }
                    } 
    
                    $order->update();
    
                    $addPayment->is_paid = 1;
                    $addPayment->paid_at = DateUtils::now();
                    $addPayment->update();

                    $payment = new Payment();
                    $payment->transaction_id = $transaction_id;
                    $payment->currency_id = $order->currency_id;
                    $payment->payment_method = ArrayUtils::get($payload, 'payment_method');
                    $payment->conversion_rate = $order->currency_to_usd_rate;
                    $payment->node_id = $order->id;
                    $payment->node_type = 'add-payment';
                    $payment->amount = $addPayment->total;
                    $payment->add();

                    $customer = new Customer( (int) $order->customer_id );
                    if(Validate::isLoadedObject($customer)) {
                        $customer->total_spent = $customer->total_spent + $addPayment->total;
                        $customer->update();
    
                        $site = new Site( (int) $customer->site_id );
                        if( Validate::isLoadedObject($site) ) {
                            $site->payments = $site->payments + $addPayment->total;
                            $site->update();
                        }
                    }

                    try { 
                        $receivers = $app->getNotificationEmails( false );
                        $writer = $order->getWriter();
                        if(Validate::isLoadedObject($writer)) {
                            $receivers[] = $writer->email;
                        }

                        foreach ($receivers as $receiver) {
                            Mail::Send(
                                MailTemplate::TEMPLATE_ORDER_INFO_UPDATE,
                                'Order Update: Please Review New Order Details #' . $order->id . ' at ' . Configuration::get('SITE_NAME', PROX_ADMIN_SITE_ID),
                                array(
                                    'order_url' => Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID) . '/order/' . $order->id
                                ),
                                $receiver,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                PROX_ADMIN_SITE_ID
                            );
                        }
                    } catch (\Exception $e) {
                    }
                }
    
                return true;
                break;

            case 'tip':
                $amount = (float) ArrayUtils::get($payload, 'amount');
                $payment_method = ArrayUtils::get($payload, 'payment_method');
                $currency_code = ArrayUtils::get($payload, 'currency'); 
                $transaction_id =ArrayUtils::get($payload, 'paymentId');

                $order = new Order( (int) $nodeId );
                if (!Validate::isLoadedObject($order)) {
                    return false;
                }

                $currencyId = Currency::getIdByIsoCode( $currency_code );
                $currency = Currency::getCurrencyInstance( (int) $currencyId);

                // check if transaction exists
                $query = new DbQuery();
                $query->select('order_tip_id');
                $query->from('order_tip');
                $query->where('transaction_id = \'' . pSQL($transaction_id) . '\'');
                $exists = Db::getInstance()->getValue($query);

                if(!$exists) {
                    $writer_tip_amount = ($amount * Configuration::get('TIP_WRITER_PERCENT', PROX_ADMIN_SITE_ID)) / 100;
                    $editor_tip_amount = ($amount * Configuration::get('TIP_EDITOR_PERCENT', PROX_ADMIN_SITE_ID)) / 100;

                    $result = Db::getInstance()->insert(
                        'order_tip',
                        array(
                            'order_id' => $order->id,
                            'currency_id' => $currency->id,
                            'transaction_id' => $transaction_id, 
                            'payment_method' => $payment_method,
                            'amount' => $amount,
                            'writer_tip_amount' => $writer_tip_amount,
                            'editor_tip_amount' => $editor_tip_amount,
                            'date_add' => DateUtils::now(),
                        )
                    );

                    if($writer_tip_amount > 0) {
                        $order->writer_pay = $order->writer_pay + $writer_tip_amount;
                    }

                    if($editor_tip_amount > 0) {
                        $order->editor_pay = $order->editor_pay + $editor_tip_amount;
                    }

                    $order->update();

                    $payment = new Payment();
                    $payment->currency_id = $order->currency_id;
                    $payment->conversion_rate = $order->currency_to_usd_rate;
                    $payment->node_id = $order->id;
                    $payment->node_type = 'tip';
                    $payment->transaction_id = $transaction_id;
                    $payment->payment_method = $payment_method;
                    $payment->amount = $amount;
                    $payment->add();
                }
                break;

            case 'wallet':
                $amount = (float) ArrayUtils::get($payload, 'amount');
                $payment_method = ArrayUtils::get($payload, 'payment_method');
                $currency_code = ArrayUtils::get($payload, 'currency'); 
                $transaction_id =ArrayUtils::get($payload, 'paymentId');

                $customer = new Customer( (int) $nodeId );
                if (!Validate::isLoadedObject($customer)) {
                    return false;
                }

                $currencyId = Currency::getIdByIsoCode( $currency_code );
                $currency = Currency::getCurrencyInstance( (int) $currencyId);

                // check if transaction exists
                $query = new DbQuery();
                $query->select('wallet_transaction_id');
                $query->from('wallet_transaction');
                $query->where('transaction_id = \'' . pSQL($transaction_id) . '\'');
                $exists = Db::getInstance()->getValue($query);

                if(!$exists) {
                    $result = Db::getInstance()->insert(
                        'wallet_transaction',
                        array(
                            'customer_id' => $customer->id,
                            'currency_id' => $currency->id,
                            'transaction_id' => $transaction_id, 
                            'source' => 'deposit',
                            'payment_method' => $payment_method,
                            'amount' => $amount,
                            'date_add' => DateUtils::now(),
                        )
                    );

                    if(!$result) {
                        return false;
                    }

                    Db::getInstance()->update(
                        'customer',
                        array(
                            'wallet_balance' => (float) $customer->wallet_balance + $amount
                        ),
                        'customer_id = ' . (int) $customer->id
                    );

                    $payment = new Payment();
                    $payment->currency_id = $currency->id;
                    $payment->conversion_rate = $currency->conversion_rate;
                    $payment->node_id = $customer->id;
                    $payment->node_type = 'wallet';
                    $payment->transaction_id = $transaction_id;
                    $payment->payment_method = $payment_method;
                    $payment->amount = $amount;
                    $payment->add();
                }

                return true;

                break;

            case 'paper':
                if(!Module::getInstanceByName('prepapers')) {
                    return false;
                }

                $customer_id = (int) ArrayUtils::get($payload, 'customer_id');
                $payment_method = ArrayUtils::get($payload, 'payment_method');
                $currency_code = ArrayUtils::get($payload, 'currency'); 
                $transaction_id =ArrayUtils::get($payload, 'paymentId');

                $prepaper = new PrePaper( (int) $nodeId );
                if (!Validate::isLoadedObject($prepaper)) {
                    return false;
                }

                $amount = $prepaper->price;

                $customer = new Customer( (int) $customer_id );
                if (!Validate::isLoadedObject($customer)) {
                    return false;
                }

                $currency = Currency::getDefaultCurrency();

                $order = new Order();
                $order->customer_id = $customer->id;
                $order->payment_method = $payment_method;
                $order->service_type = Order::COMPLEX_ASSIGNMENT;
                $order->currency_id = $currency->id;
                $order->currency_to_usd_rate = $currency->conversion_rate;
                $order->status_id = Order::FINISHED;
                $order->title = $prepaper->title;
                $order->academic_level_id = $prepaper->academic_level_id;
                $order->paper_type_id = $prepaper->paper_type_id;
                $order->paper_type_option = $prepaper->paper_type_option;
                $order->paper_format_id = $prepaper->paper_format_id;
                $order->paper_format_option = $prepaper->paper_format_option;
                $order->topic_category_id = $prepaper->topic_category_id;
                $order->topic_category_option = $prepaper->topic_category_option;
                $order->is_complex_assignment = $prepaper->is_complex_assignment;

                $academicDeadlines = Deadline::getAcademicDeadlines( $customer->site_id );
                $academicDeadline = $academicDeadlines[0];

                $order->tariff_id = $academicDeadline['deadline_id'];
                $order->price_per_page = $academicDeadline['price_per_page'];
                $order->hrs_customer = $academicDeadline['hrs_customer'];
                $order->hrs_writer = $academicDeadline['hrs_writer'];
                $order->customer_deadline = DateUtils::now();
                $order->writer_deadline = DateUtils::now();
  
                $order->pages = $prepaper->pages;
                $order->charts = $prepaper->charts;
                $order->slides = $prepaper->slides;
                $order->words = $prepaper->words;
                $order->sources = $prepaper->sources;
                $order->spacing = $prepaper->spacing;
                $order->paper_details = $prepaper->question;
                $order->raw_cost = $prepaper->price;
                $order->total = $prepaper->price;
                $order->total_paid = $prepaper->price;
                $order->is_paid = true;
                $order->paid_at = DateUtils::now();
                $order->completed_at = DateUtils::now();
                $order->delivered_at = DateUtils::now();
                $order->finished_at = DateUtils::now();
                $order->add();
                
                # insert order files
                $prepaperFiles = $prepaper->getFiles();

                foreach($prepaperFiles as $prepaperFile) {
                    $prepaperFile = new PrePaperFile( (int) $prepaperFile['prepaper_file_id'] );
                    if(Validate::isLoadedObject($prepaperFile)) {
                        $orderFile = new File();
                        $orderFile->order_id = $order->id;
                        $orderFile->site_id = $customer->site_id;
                        $orderFile->file_type_id = File::FINAL_PAPER;
                        $orderFile->file_description = 'Final Paper';
                        $orderFile->name = $prepaperFile->name;
                        $orderFile->size = $prepaperFile->size;
                        $orderFile->source = $prepaperFile->source;
                        $orderFile->extension = $prepaperFile->extension;
                        $orderFile->is_final_file = true;
                        $orderFile->add();
                    }
                }
            
                $ok = Db::getInstance()->insert(
                    'prepaper_payment',
                    array(
                        'prepaper_id' => $prepaper->id,
                        'customer_id' => $customer->id,
                        'order_id' => $order->id,
                        'payment_token' => $transaction_id, 
                        'payment_method' => $payment_method,
                        'amount' => $amount,
                        'is_paid' => true,
                        'paid_at' => DateUtils::now(),
                        'date_add' => DateUtils::now(),
                    )
                );

                if(!$ok) {
                    return false;
                }

                $payment = new Payment();
                $payment->currency_id = $currency->id;
                $payment->conversion_rate = $currency->conversion_rate;
                $payment->node_id = $prepaper->id;
                $payment->node_type = 'paper';
                $payment->transaction_id = $transaction_id;
                $payment->payment_method = $payment_method;
                $payment->amount = $amount;
                $payment->add();

                return true;
                break;
        }
    }
}