<?php

/**
 * @package    Proxim
 * @author     Davison Pro <davisonpro.coder@gmail.com | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.0.0
 * @since      File available since Release 1.0.0
 */

namespace Proxim\Api;

use Db;
use Proxim\Route;
use Proxim\Configuration;
use Proxim\Currency;
use Proxim\Database\DbQuery;
use Proxim\Preference\AcademicLevel;
use Proxim\Preference\Deadline;
use Proxim\Preference\Discipline;
use Proxim\Preference\ItemPricing;
use Proxim\Preference\PaperType;
use Proxim\Presenter\Deadline\DeadlinePresenter;
use Proxim\Site\Site;
use Proxim\Validate;
use Proxim\Util\ArrayUtils;

class Calculator extends Route
{
	public function getCalculationPrices()
	{
		global $globals;
		$app = $this->app;
		$smarty = $app->container->smarty;
		$params = $app->request->get();

		$configsForCalculations = $configsForProgramming = array();

		# programming & calculations
		$configTaskSizes = array();
		$taskSizes = ArrayUtils::get($globals, 'taskSizes');

		foreach ($taskSizes as $taskSizeId => $taskSize) {
			$configTaskSizes[] = array(
				'id' => (int) $taskSizeId,
				'title' => $taskSize['title']
			);
		}

		$configsForCalculations['sizes'] = $configsForProgramming['sizes'] = $configTaskSizes;

		# calculations
		$calculationsDisciplines = Discipline::getCalculationsDisciplines();

		$calculationsItemPricing = array();
		foreach ($calculationsDisciplines as $discipline) {
			$sql = new DbQuery();
			$sql->select('ip.item_pricing_id');
			$sql->from('item_pricing', 'ip');
			$sql->where('ip.discipline_id = ' . (int) $discipline['discipline_id']);
			$sql->orderBy('ip.task_size_id ASC');
			$itemPricing = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);

			foreach ($itemPricing as $itemPrice) {
				$itemPricing = new ItemPricing((int) $itemPrice['item_pricing_id']);
				if (Validate::isLoadedObject($itemPricing)) {
					$calculationsItemPricing[] = array(
						'sizeId' => (int) $itemPricing->task_size_id,
						'price' => (float) $itemPricing->price
					);
				}
			}
		}
		
		$calculationsTaskSizePrices = [];
		foreach ($calculationsItemPricing as $pricing) {
			$sizeId = $pricing['sizeId'];
			$calculationsTaskSizePrices[$sizeId]['sum'] = ($calculationsTaskSizePrices[$sizeId]['sum'] ?? 0) + $pricing['price'];
			$calculationsTaskSizePrices[$sizeId]['count'] = ($calculationsTaskSizePrices[$sizeId]['count'] ?? 0) + 1;
		}

		$calculationsTaskSizePrices = array_map(fn($item) => $item['sum'] / $item['count'], $calculationsTaskSizePrices);
		ksort($calculationsTaskSizePrices);

		$calculationsTariffs = Deadline::getCalculationsDeadlines(PROX_SITE_ID);
		$calculationsDeadlines = array();
		foreach ($calculationsTariffs as $deadline) {
			$tariff = new Deadline((int) $deadline['deadline_id']);
			if (Validate::isLoadedObject($tariff)) {

				foreach($calculationsTaskSizePrices as $taskSizeId => $taskSizePrice) {
					$calculationsDeadlines[$tariff->name][$taskSizeId] = array(
						'pricePerPage' => (float) $taskSizePrice * (float) $tariff->price_modifier
					);
				}
			}
		}

		$configsForCalculations['deadlines'] = $calculationsDeadlines;

		# programming
		$programmingDisciplines = Discipline::getProgrammingDisciplines();

		$programmingItemPricing = array();
		foreach ($programmingDisciplines as $discipline) {
			$sql = new DbQuery();
			$sql->select('ip.item_pricing_id');
			$sql->from('item_pricing', 'ip');
			$sql->where('ip.discipline_id = ' . (int) $discipline['discipline_id']);
			$sql->orderBy('ip.task_size_id ASC');
			$itemPricing = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);

			foreach ($itemPricing as $itemPrice) {
				$itemPricing = new ItemPricing((int) $itemPrice['item_pricing_id']);
				if (Validate::isLoadedObject($itemPricing)) {
					$programmingItemPricing[] = array(
						'sizeId' => (int) $itemPricing->task_size_id,
						'price' => (float) $itemPricing->price
					);
				}
			}
		}

		$programmingTaskSizePrices = [];
		foreach ($programmingItemPricing as $pricing) {
			$sizeId = $pricing['sizeId'];
			$programmingTaskSizePrices[$sizeId]['sum'] = ($programmingTaskSizePrices[$sizeId]['sum'] ?? 0) + $pricing['price'];
			$programmingTaskSizePrices[$sizeId]['count'] = ($programmingTaskSizePrices[$sizeId]['count'] ?? 0) + 1;
		}

		$programmingTaskSizePrices = array_map(fn($item) => $item['sum'] / $item['count'], $programmingTaskSizePrices);
		ksort($programmingTaskSizePrices);

		$programmingTariffs = Deadline::getProgrammingDeadlines(PROX_SITE_ID);
		$programmingDeadlines = array();
		foreach ($programmingTariffs as $deadline) {
			$tariff = new Deadline((int) $deadline['deadline_id']);
			if (Validate::isLoadedObject($tariff)) {

				foreach($programmingTaskSizePrices as $taskSizeId => $taskSizePrice) {
					$programmingDeadlines[$tariff->name][$taskSizeId] = array(
						'pricePerPage' => (float) $taskSizePrice * (float) $tariff->price_modifier
					);
				}
			}
		}

		$configsForProgramming['deadlines'] = $programmingDeadlines;

		return $app->response([
			"configsForCalculations" => $configsForCalculations,
			"configsForProgramming" => $configsForProgramming
		]);
	}

	public function getCalculator()
	{
		global $globals;
		$app = $this->app;
		$smarty = $app->container->smarty;
		$params = $app->request->get();
		$options = ArrayUtils::get($params, 'options', []);
		$format = ArrayUtils::get($params, 'format', 'html');

		// default options
		$deafault = ArrayUtils::get($options, 'deafault', []);

		$academicLevelDefault = Configuration::get('DEFAULT_COURSELEVEL', null, 2);
		$paperTypesDefault = ArrayUtils::get($deafault, 'paperTypesDefault', 2);
		$pagesDefault = ArrayUtils::get($deafault, 'pagesDefault', 1);

		$pricingForm = $smarty->fetch("ajax.pricingform.tpl");

		$sitePaperTypes = PaperType::getPaperTypes();

		$paperTypes = array();
		foreach ($sitePaperTypes as $paperType) {
			$paperTypes[] = array(
				'id' => (int) $paperType['paper_type_id'],
				'option' => $paperType['title'],
				'enabled' => true,
				'isDissertation' => false,
			);
		}

		$defaultTariffs = array();
		$default_deadlines = (string) Configuration::get('DEFAULT_DEADLINES');
		$default_deadlines = explode(',', $default_deadlines);

		foreach ($default_deadlines as $deadline_id) {
			$defaultTariffs[] = (int) $deadline_id;
		}

		$deadlines = Deadline::getAcademicDeadlines(PROX_SITE_ID);

		$tariffs = array();

		$deadlinePresenter = new DeadlinePresenter();
		foreach ($deadlines as $deadline) {
			$deadline = new Deadline($deadline['deadline_id']);
			if (Validate::isLoadedObject($deadline)) {
				$deadline = $deadlinePresenter->present($deadline);
				$tariffs[] = [
					"id" => (int) $deadline['id'],
					"name" => $deadline['name'],
					"pricePerPage" => (float) $deadline['pricePerPage'],
					"minPages" => (int) $deadline['minPages'],
					"hrs" => (int) $deadline['hrs'],
					"hrsWriter" => (int) $deadline['hrsWriter'],
					"academicLevel" =>  (int) $deadline['academicLevel'],
					"isDefault" => in_array($deadline['id'], $defaultTariffs) ? true : false
				];
			}
		}

		$academicLevels = AcademicLevel::getAcademicLevels();

		$currencies = Currency::getCurrencies();

		$availableCurrencies = array();
		foreach ($currencies as $currency) {
			$availableCurrencies[$currency['iso_code']] = array(
				"symbol" => $currency['symbol'],
				"rate" => (float) $currency['conversion_rate'],
			);
		}

		$defaultCurrencyISO = "USD";
		$defaultCurrencyID = Configuration::get("CURRENCY_DEFAULT");
		$defaultCurrency = new Currency((int) $defaultCurrencyID);
		if (Validate::isLoadedObject($defaultCurrency)) {
			$defaultCurrencyISO = $defaultCurrency->iso_code;
		}

		$paperTypesReturn = $academicLevelsReturn = array();

		$paperTypesReturn[] = $paperTypes;
		$paperTypesReturn['paperTypeDefault'] = $paperTypesDefault;

		$academicLevelsReturn[] = $academicLevels;
		$academicLevelsReturn['academicLevelDefault'] = $academicLevelDefault;

		return $app->response([
			"html" => $pricingForm,
			"siteId" => Configuration::get("SITE_ID"),
			"siteUrl" => Configuration::get("SITE_DOMAIN"),
			"paperTypes" => $paperTypesReturn,
			"dissertationPaperTypes" => array(),
			"academicLevels" => $academicLevelsReturn,
			"deadlines" => $tariffs,
			"pages" => array(
				"defaultPages" => $pagesDefault
			),
			"environment" => "production",
			"currency" => [
				"defaultCurrency" => $defaultCurrencyISO,
				"currencyInfo" => $availableCurrencies
			]
		]);
	}

	public function getPricingBlock()
	{
		global $globals;
		$app = $this->app;
		$smarty = $app->container->smarty;
		$params = $app->request->get();
		$format = ArrayUtils::get($params, 'format', 'html');

		$academicLevels = AcademicLevel::getSiteAcademicLevels();

		$deadlinesName = array();

		$deadlines = Deadline::getAcademicDeadlines(PROX_SITE_ID);
		$tariffs = array();

		$deadlinePresenter = new DeadlinePresenter();
		foreach ($deadlines as $deadline) {
			$deadline = new Deadline($deadline['deadline_id']);
			if (Validate::isLoadedObject($deadline)) {
				$deadline = $deadlinePresenter->present($deadline);

				$tariffs[] = [
					"id" => (int) $deadline['id'],
					"name" => $deadline['name'],
					"pricePerPage" => (float) $deadline['pricePerPage'],
					"minPages" => (int) $deadline['minPages'],
					"hrs" => (int) $deadline['hrs'],
					"hrsWriter" => (int) $deadline['hrsWriter'],
					"academicLevel" =>  (int) $deadline['academicLevel'],
				];
			}
		}

		foreach ($tariffs as $value) {
			if (is_array($value) && array_key_exists("name", $value)) {
				$deadlinesName[$value["name"]][$value["academicLevel"]] = $value;
			}
		}

		$currencies = Currency::getCurrencies();

		$availableCurrencies = array();
		foreach ($currencies as $currency) {
			$availableCurrencies[$currency['iso_code']] = array(
				"symbol" => $currency['symbol'],
				"rate" => (float) $currency['conversion_rate'],
			);
		}

		$defaultCurrencyISO = "USD";
		$defaultCurrencyID = Configuration::get("CURRENCY_DEFAULT");
		$defaultCurrency = new Currency((int) $defaultCurrencyID);
		if (Validate::isLoadedObject($defaultCurrency)) {
			$defaultCurrencyISO = $defaultCurrency->iso_code;
		}

		$deadlinesName = array_reverse($deadlinesName);

		$return = [
			"environment" => "production",
			"currency" => [
				"defaultCurrency" => $defaultCurrencyISO,
				"currencyInfo" => $availableCurrencies
			],
			"deadlines" => $deadlinesName,
			"academicLevels" => $academicLevels
		];

		if ($format == 'json') {
			return $app->response($return);
		} else {
			$smarty->assign($return);

			$app->setTemplate('ajax.pricingtable');
			return $app->display();
		}
	}

	public function disciplinesStatistics()
	{
		global $globals;
		$app = $this->app;
		$smarty = $app->container->smarty;
		$params = $app->request->get();
		$academicLevelStat = (int) ArrayUtils::get($params, 'academicLevel', 1);
		$disciplineStat = (int) ArrayUtils::get($params, 'discipline', 1);

		$academicLevels = AcademicLevel::getSiteAcademicLevels();

		$academicLevelsOptions = array();
		foreach ($academicLevels as $academicLevel) {
			$academicLevelsOptions[] = array(
				'id' => (int) $academicLevel['id'],
				'option' => $academicLevel['title'],
			);
		}

		$disciplineGroups = Discipline::getDisciplinesByGroup(true);

		$disciplines = array();
		foreach ($disciplineGroups as $disciplineGroup) {

			$disciplinesX = array();
			if (count($disciplineGroup['disciplines'])) {
				foreach ($disciplineGroup['disciplines'] as $discipline) {
					$disciplinesX[] = [
						'id' => (int) $discipline['id'],
						'name' => $discipline['name']
					];
				}

				$disciplines[$disciplineGroup['name']] = $disciplinesX;
			}
		}

		$sql = new DbQuery();
		$sql->select('SUM(paper_rating+service_rating) AS paper_rating, COUNT(*) total_ratings');
		$sql->from('order_rating', 'orating');
		$sql->innerJoin('order', 'o', 'orating.order_id = o.order_id');
		$sql->where('o.academic_level_id = ' . (int) $academicLevelStat);
		$sql->where('o.topic_category_id = ' . (int) $disciplineStat);

		$results = Db::getInstance()->getRow($sql);

		$paper_rating = 0;
		if ($results['paper_rating'] && $results['total_ratings']) {
			$paper_rating = $results['paper_rating'] / 2;
			$averageStats = number_format(($paper_rating / $results['total_ratings']), 2);
		} else {
			if ($results['paper_rating']) {
				$paper_rating = $results['paper_rating'] / 2;
			}
			$averageStats = number_format($paper_rating, 2);
		}

		$totalSum = $results['total_ratings'];

		$return = array(
			'success' => true,
			'ratings' => [
				'academicLevels' => $academicLevelsOptions,
				'disciplines' => $disciplines,
				"statisticBySite" => [
					"average" => (float) $averageStats,
					"sum" => (int) $totalSum
				],
				"statisticGlobal" => [
					"average" => (float) $averageStats,
					"sum" => (int) $totalSum
				]
			]
		);

		return $app->response($return);
	}
}
