<?php 
/**
 * @package    Proxim
 * @author     Davison Pro <davisonpro.coder@gmail.com | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.0.0
 * @since      File available since Release 1.0.0
 */

namespace Proxim\Api;

use Db;
use Proxim\Route;
use Proxim\Tools;
use Proxim\Configuration;
use Proxim\Currency;
use Proxim\Database\DbQuery;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Order\Order;
use Proxim\Order\Rating;
use Proxim\Preference\AcademicLevel;
use Proxim\Util\ArrayUtils;
use Proxim\Presenter\Deadline\DeadlinePresenter;
use Proxim\Preference\UploadType;
use Proxim\Preference\PaperType;
use Proxim\Preference\Deadline;
use Proxim\Preference\Deliverable;
use Proxim\Preference\Descriptor;
use Proxim\Preference\Discipline;
use Proxim\Preference\ItemPricing;
use Proxim\Preference\PaperFormat;
use Proxim\Presenter\Order\OrderRatingPresenter;
use Proxim\Util\DateUtils;
use Proxim\Validate;

class Config extends Route {

    public function getConfig() {
        global $globals;
        $app = $this->app;
		$user = $app->container->user;
        $site = $app->container->site;

		$configsForAcademicWriting = $configsForCalculations = $configsForProgramming = $configsForArticleWriting = array();

		$isProgrammingEnabled = Configuration::get('PROGRAMMING_ENABLED', $site->id, false);
		$isCalculationsEnabled = Configuration::get('CALCULATIONS_ENABLED', $site->id, false);
		$isAcademicWritingEnabled = Configuration::get('ACADEMIC_WRITING_ENABLED', $site->id, true);
		$isArticleWritingEnabled = Configuration::get('ARTICLE_WRITING_ENABLED', $site->id, false);
		$isAnonymousEnabled = Configuration::get('ANONYMOUS_ENABLED', $site->id, false);

		$return = array(
			"siteFullName" => Configuration::get('SITE_NAME'),
			"siteUrlSsl" => Configuration::get('SITE_DOMAIN', PROX_SITE_ID, $app->base_uri),
            "siteCurrencyDefault" => (int) Configuration::get('CURRENCY_DEFAULT', $site->id, 1),

			'isAnonymousEnabled' => (bool) $isAnonymousEnabled,

			'isProgrammingEnabled' => (bool) $isProgrammingEnabled,
			'isCalculationsEnabled' => (bool) $isCalculationsEnabled,
			'isAcademicWritingEnabled' => (bool) $isAcademicWritingEnabled,
			'isArticleWritingEnabled' => (bool) $isArticleWritingEnabled,
			// thirdparty signin
			'isGoogleSignInEnabled' => (bool) Configuration::get('GOOGLE_SIGNIN_ENABLED', $site->id, false),
			'isAppleSignInEnabled' => (bool) Configuration::get('APPLE_SIGNIN_ENABLED', $site->id, false),
			// onboarding form
			'isOnboardingEnabled' => (bool) Configuration::get('ONBOARDING_FORM_ENABLED', $site->id, true),

			'isReferralProgramEnabled' => (bool) Configuration::get('REWARDS_PROGRAM_ENABLED', PROX_ADMIN_SITE_ID, false),
			'rewardsProgramDiscount' => (float) Configuration::get('REWARDS_PROGRAM_DISCOUNT', PROX_ADMIN_SITE_ID, 0),
			"walletEnabled" => (bool) Configuration::get('WALLET_ENABLED', PROX_ADMIN_SITE_ID, false),
			"leaveTipEnabled" => (bool) Configuration::get('LEAVE_TIP_ENABLED', PROX_ADMIN_SITE_ID, false),
            "defaultCourselevelId" => (int) Configuration::get('DEFAULT_COURSELEVEL', $site->id, 2),
            "hasDiscounts" => (bool) Configuration::get('DISCOUNTS_ENABLED', $site->id, false),
			"hasInquiry" => (bool) Configuration::get('INQUIRY_ENABLED', $site->id, false),
			"orderFormCoupons" => (bool) Configuration::get('ORDER_FORM_COUPONS', $site->id, false),
			"expertProofreading" => (bool) Configuration::get('EXPERT_PROOFREADING_ENABLED', $site->id, false),
            "vipSupport" => (bool) Configuration::get('VIP_SUPPORT_ENABLED', $site->id, false),
            "plagiarismReport" => (bool) Configuration::get('PLAGIARISM_REPORT_ENABLED', $site->id, false),
            "draftOutline" => (bool) Configuration::get('DRAFT_OUTLINE_ENABLED', $site->id, false),
            "writerSamples" => (bool) Configuration::get('WRITER_SAMPLES_ENABLED', $site->id, false),
            "progressiveDelivery" => (bool) Configuration::get('PROGRESSIVE_DELIVERY_ENABLED', $site->id, false),
            "copyOfSources" => (bool) Configuration::get('COPY_OF_SOURCES_ENABLED', $site->id, false),
			"wordsPerPage" => (int) Configuration::get('WORDS_PER_PAGE', PROX_ADMIN_SITE_ID, 275),
			"urgentDeadlinesMaxPages" => (int) Configuration::get('URGENT_DEADLINE_MAX_PAGES', $site->id, 2)
		);

		$isTrustpilotEnabled = Configuration::get('TRUSTPILOT_ENABLED', PROX_ADMIN_SITE_ID, false);
		$return['isTrustpilotEnabled'] = (bool) $isTrustpilotEnabled;

		$reviews = $trustpilotReviews = array();

		if($isTrustpilotEnabled) {
			$return['tpRatingTitle'] = Configuration::get('TRUSTPILOT_TITLE', PROX_ADMIN_SITE_ID);
			$return['tpTotalReviews'] = Configuration::get('TRUSTPILOT_REVIEWS', PROX_ADMIN_SITE_ID);
			$return['tpRatingStars'] = Configuration::get('TRUSTPILOT_STARS', PROX_ADMIN_SITE_ID);
			$return['tpProfileLink'] = Configuration::get('TRUSTPILOT_LINK', PROX_ADMIN_SITE_ID);
			
			if(Configuration::get('TRUSTPILOT_SHOW_REVIEWS', PROX_ADMIN_SITE_ID)) {
				$sql = new DbQuery();
				$sql->select('review.*');
				$sql->from('trustpilot_review', 'review');
				$sql->orderBy('review.review_id DESC');
				$sql->orderBy('RAND()');
				$sql->limit(10);
				$results = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);
	
				foreach( $results as $review ) {
					$body_formatted = Tools::getStringClean($review['body']);
					$body_formatted = Tools::truncateString($body_formatted, 150);
					
					$trustpilotReviews[] = [
						'id' => (int) $review['review_id'],
						'name' => $review['author_name'],
						'title' => $review['title'],
						'body' => $body_formatted,
						'rating' => (float) number_format($review['stars'], 1),
						'createdAt' => DateUtils::convertToISOFormat($review['time'])
					];
				}
			} else {
				$sql = new DbQuery();
				$sql->select('r.rating_id');
				$sql->from('order_rating', 'r');
				$sql->innerJoin('order', 'o', 'r.`order_id` = o.`order_id`');
				$sql->where('o.service_type = ' . Order::COMPLEX_ASSIGNMENT);
				$sql->where('r.paper_rating >= ' . 8);
				$sql->where("r.comment <> ''"); 
				$sql->where("YEAR(r.date_add) = " . date('Y')); 
				$sql->orderBy('RAND()');
				$sql->limit(10);
				$results = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);

				$ratingPresenter = new OrderRatingPresenter();
				foreach( $results as $rating ) {
					$rating = new Rating( (int) $rating['rating_id'] );
					if(Validate::isLoadedObject($rating)) {
						$rating = $ratingPresenter->present($rating);
						$reviews[] = $rating;
					}
				}
			}
		}

		$return['reviews'] = $reviews;
		$return['trustpilotReviews'] = $trustpilotReviews;

		$currencies = Currency::getCurrencies( true );
		
        $availableCurrencies = array();
		foreach ($currencies as $currency) {
			$availableCurrencies[] = array(
				"title" => ArrayUtils::get($currency, 'title'),
				"currencyId" => (int) ArrayUtils::get($currency, 'currency_id'),
				"symbols" => ArrayUtils::get($currency, 'symbol'),
				"exchangeRateData" => array(
					"currency" => ArrayUtils::get($currency, 'iso_code'),
					"rate" => (float) ArrayUtils::get($currency, 'conversion_rate'),
					"isInverted" => false
				)
			);
		}

		$return["siteCurrencies"] = $availableCurrencies;

        $paperTypes = $paperFormats = $academicLevels = $writerPreferences = $orderServiceTypes = $departments = array();

        $revisionDeadlines = array();
		foreach ( $globals['revisionDeadlines'] as $deadlineId => $deadline ) {
			$deadline['id'] = (int) $deadlineId;
			$revisionDeadlines[] = $deadline;
		}

		$paidRevisionDeadlines = array();
		foreach ( $globals['paidRevisionDeadlines'] as $serviceTypeId => $serviceTypeDeadlines ) {
			$serviceDeadlines = array();
			foreach( $serviceTypeDeadlines as $paidDeadlineId => $paidDeadlines ) {
				$deadlines = array();
				foreach( $paidDeadlines as $deadlineId => $deadline ) {
					$deadline['did'] = (int) $deadlineId;
					$deadlines[] = $deadline;
				}
				$serviceDeadlines[$paidDeadlineId] = $deadlines;
			}

			$paidRevisionDeadlines[$serviceTypeId] = $serviceDeadlines;
		}

		$return['revisionDeadlines'] = $revisionDeadlines;
		$return['paidRevisionDeadlines'] = $paidRevisionDeadlines;

		foreach ( $globals['departments'] as $departmentId => $department ) {
			$department['id'] = (int) $departmentId;
			$departments[] = $department;
		}

		// writer preferences
		foreach ( $globals['writerPreferences'] as $writerPreferenceId => $writerPreference ) {
			$writerPreference['id'] = (int) $writerPreferenceId;
			$writerPreferences[] = $writerPreference;
		}	

		$return['writerPreferences'] = $writerPreferences;

		// service types
		foreach ( $globals['serviceTypes'] as $orderServiceTypeId => $orderServiceType ) {
			$orderServiceType['id'] = (int) $orderServiceTypeId;
			$orderServiceTypes[] = $orderServiceType;
		}	

		$return["orderServiceTypes"] = $orderServiceTypes;
 
		$deliverablesDictionary = array();
		$deliverables = Deliverable::getDeliverables();

		foreach($deliverables as $deliverable) {
			$deliverablesDictionary[] = array(
				'id' => (int) $deliverable['deliverable_id'],
				'name' => $deliverable['name'],
				'slug' => $deliverable['slug']
			);
		}

		$return["deliverablesDictionary"] = $deliverablesDictionary;

		$rawUploadTypes = UploadType::getUploadTypes();

		$uploadTypes = array();
		foreach( $rawUploadTypes as $uploadType ) {
			if( $uploadType['customer'] ) {
				$uploadTypes[] = array(
					'id' => (int) $uploadType['upload_type_id'],
					'description' => $uploadType['title']
				);
			}
		}

		$return['uploadTypes'] = $uploadTypes;

		/** Countries **/
		$taxRates = array();
		if(Configuration::get('TAX_RATES_ENABLED', PROX_ADMIN_SITE_ID, false)) {
			$taxRates = Configuration::get('TAX_RATES_RULES', PROX_ADMIN_SITE_ID, array());
			$taxRates = json_decode($taxRates, true);
		}

		$countries = array();
		$i = 0;
		foreach ($globals['countries'] as $key => $country) {
			$country_code = Tools::strtoupper($key);
			$country['index'] = $i;
			$country['id'] = $key;
			
			if(ArrayUtils::has($taxRates, $country_code)) {
				$taxRate = ArrayUtils::get($taxRates, $country_code, array());
				$country['vat'] = (float) ArrayUtils::get($taxRate, 'rate', 0);
			}

			$countries[] = $country;
			$i++;
        }	

		/** Payment methods **/
		$paymentModules = PaymentModule::getInstalledPaymentModules();
		$paymentMethods = array();
		foreach($paymentModules as $paymentModule) {
			$paymentModule = Module::getInstanceByName($paymentModule['name']);
			if(Validate::isLoadedObject($paymentModule)) {
				if($paymentModule->isActive()) {
					$paymentDisplayName = Configuration::get($paymentModule->name . '_custom_name', PROX_SITE_ID, $paymentModule->displayName);

					$paymentMethods[] = array(
						"enabled" => true,
						"name" => $paymentDisplayName,
						"value" => $paymentModule->name,
						"availableCountries" => null
					);
				}
			}
		}

		$defaultPaymentMethod = Configuration::get('PAYMENT_METHOD_DEFAULT');
		if(!$defaultPaymentMethod && count($paymentMethods)) {
			$defaultPaymentMethod = $paymentMethods[0]['value'];
		}

		if($user->isLogged() && Configuration::get('WALLET_ENABLED', PROX_ADMIN_SITE_ID)) {
			$paymentMethods[] = array(
				"enabled" => true,
				"name" => 'My Wallet',
				"value" => 'wallet',
				"availableCountries" => null
			);
		}

		$return['paymentMethods'] = $paymentMethods;

		# configs for academic writing
		$sitePaperFormats = PaperFormat::getPaperFormats();
		foreach ( $sitePaperFormats as $paperFormat ) {
			$paperFormats[] = array(
				'id' => (int) $paperFormat['paper_format_id'],
				'title' => $paperFormat['title']
			);
		}

		$configsForAcademicWriting['paperFormats'] = $paperFormats;

		$academicWritingPaperTypes = PaperType::getPaperTypes( Order::COMPLEX_ASSIGNMENT );

		//paper types
		$configAcademicWritingPaperTypes = array();
		foreach ( $academicWritingPaperTypes as $paperType ) {
			$configAcademicWritingPaperTypes[] = array(
				'id' => (int) $paperType['paper_type_id'],
				'title' => $paperType['title']
			);
		}

		$configsForAcademicWriting['paperTypes'] = $configAcademicWritingPaperTypes;

		// academic levels
		$academicLevels = AcademicLevel::getSiteAcademicLevels();
		$configsForAcademicWriting['academicLevels'] = $academicLevels;

		$deadlinePresenter = new DeadlinePresenter();
		$deadlines = Deadline::getAcademicDeadlines( $site->id );

		$tariffs = array();
		foreach($deadlines as $deadline) {
			$tariff = new Deadline( (int) $deadline['deadline_id'] );
			if(Validate::isLoadedObject($tariff)) {
				$tariffs[] = $deadlinePresenter->present($tariff);
			}
		}

		$configsForAcademicWriting['tariffs'] = $tariffs;

		$defaultTariffs = [];
		$default_deadlines = (string) Configuration::get('DEFAULT_DEADLINES');
		$default_deadlines = explode(',', $default_deadlines);
		if(is_array($default_deadlines)) {
			foreach($default_deadlines as $deadline_id) {
				$defaultTariffs[] = (int) $deadline_id;
			}
		}

		$configsForAcademicWriting['defaults'] = array(
			'defaultAcademicLevelId' => (int) Configuration::get('DEFAULT_COURSELEVEL', $site->id, LEVEL_COLLEGE),
			'tariffIds' => $defaultTariffs,
		);

		$disciplines = array();

		$orderDisciplines = Discipline::getDisciplines();
		foreach( $orderDisciplines as $discipline ) {
			$disciplines[] = array(
				'id' => (int) $discipline['discipline_id'],
				'groupId' => (int) $discipline['group_id'],
				'name' => $discipline['title'],
				'isComplexAssignment' => (bool) $discipline['is_complex_assignment']
			);
		}

		$return['disciplines'] = $disciplines;

		$disciplineGroups = Discipline::getDisciplinesByGroup( true );
		$return['groupsOfDisciplinesById'] = $disciplineGroups;

		$disciplineGroupIds = [];
		foreach($disciplineGroups as $discipline) {
			$disciplineGroupIds[] = $discipline['id'];
		}

		$academicWritingDisciplines = Discipline::getAcademicWritingDisciplines();
		$disciplinesIds = array();
		foreach($academicWritingDisciplines as $discipline) {
			if(!in_array($discipline['discipline_id'], $disciplineGroupIds)) {
				$disciplinesIds[] = (int) $discipline['discipline_id'];
			} 
		}

		$configsForAcademicWriting['disciplinesIds'] = $disciplinesIds;
		
		// Top Paper Types
		$getAcademicTopPaperTypes = PaperType::getTopPaperTypes( 5, Order::COMPLEX_ASSIGNMENT );
		$topAcademicPaperTypes = array();
		foreach( $getAcademicTopPaperTypes as $paperType ) {
			$paperType = new PaperType( (int) $paperType['paper_type_id'] );
			if(Validate::isLoadedObject($paperType)) {
				$topAcademicPaperTypes[] = (int) $paperType->id;
			}
		}

		$configsForAcademicWriting['topPaperTypes'] = $topAcademicPaperTypes;

		// Top Disciplines
		$getTopDisciplines = Discipline::getTopDisciplines( 5, Order::COMPLEX_ASSIGNMENT );
		$topDisciplines = array();
		foreach( $getTopDisciplines as $discipline ) {
			$discipline = new Discipline( (int) $discipline['topic_category_id'] );
			if(Validate::isLoadedObject($discipline)) {
				$topDisciplines[] = (int) $discipline->id;
			}
		}

		$configsForAcademicWriting['topDisciplinesIds'] = $topDisciplines;

		
		# programming & calculations
		$configTaskSizes = array();
		$taskSizes = ArrayUtils::get($globals, 'taskSizes');

		foreach($taskSizes as $taskSizeId => $taskSize) {
			$configTaskSizes[] = array(
				'id' => (int) $taskSizeId,
				'name' => $taskSize['title']
			);
		}
		
		$configsForCalculations['sizes'] = $configsForProgramming['sizes'] = $configTaskSizes;

		$programmingTariffs = Deadline::getProgrammingDeadlines( $site->id );
		$programmingDeadlines = array();
		foreach($programmingTariffs as $deadline) {
			$tariff = new Deadline( (int) $deadline['deadline_id'] );
			if(Validate::isLoadedObject($tariff)) {
				$programmingDeadlines[] = array(
					'id' => (int) $tariff->id, 
					'hours' => (int) $tariff->hrs_customer, 
					'name' => $tariff->name,
					'priceModifier' => (float) $tariff->price_modifier
				);
			}
		}

		$configsForProgramming['deadlines'] = $programmingDeadlines;

		$programmingDisciplines = Discipline::getProgrammingDisciplines();

		$programmingDescriptors = array();
		foreach($programmingDisciplines as $discipline) {
			$sql = new DbQuery();
			$sql->select('d.descriptor_id, d.discipline_id, d.task_size_id, d.example');
			$sql->from('descriptor', 'd');
			$sql->where('d.is_programming = 1');
			$sql->where('d.discipline_id = ' . (int) $discipline['discipline_id']);
			$sql->orderBy('d.task_size_id ASC');
			$descriptors = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);

			foreach($descriptors as $descriptor) {
				$descriptor = new Descriptor( (int) $descriptor['descriptor_id'] );
				if(Validate::isLoadedObject($descriptor)) {
					$deliverables = $descriptor->getDeliverables();
					$deliverablesIds = array();
					foreach($deliverables as $deliverable) {
						$deliverablesIds[] = (int) $deliverable['deliverable_id'];
					}
				   
					$programmingDescriptors[] = array(
						'id' => (int) $descriptor->id,
						'disciplineId' => (int) $descriptor->discipline_id,
						'taskSizeId' => (int) $descriptor->task_size_id,
						'example' => $descriptor->example,
						'deliverablesIds' => $deliverablesIds
					);
				}
			}
		}

		$configsForProgramming['descriptors'] = $programmingDescriptors;

		$programmingDisciplineIds = array();
		foreach($programmingDisciplines as $discipline) {
			$programmingDisciplineIds[] = (int) $discipline['discipline_id'];
		}

		$configsForProgramming['disciplinesIds'] = $programmingDisciplineIds;

		$programmingItemPricing = array();
		foreach($programmingDisciplines as $discipline) {
			$sql = new DbQuery();
			$sql->select('ip.item_pricing_id');
			$sql->from('item_pricing', 'ip');
			$sql->where('ip.discipline_id = ' . (int) $discipline['discipline_id'] );
			$sql->orderBy('ip.task_size_id ASC');
			$itemPricing = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);

			foreach($itemPricing as $itemPrice) {
				$itemPricing = new ItemPricing( (int) $itemPrice['item_pricing_id'] );
				if(Validate::isLoadedObject($itemPricing)) {
					$programmingItemPricing[] = array(
						'id' => (int) $itemPricing->id,
						'disciplineId' => (int) $discipline['discipline_id'],
						'sizeId' => (int) $itemPricing->task_size_id,
						'price' => (float) $itemPricing->price
					);
				}
			}
		}

		$configsForProgramming['itemPrices'] = $programmingItemPricing;

		$configsForProgramming['defaults'] = array();
		if(Configuration::get('DEFAULT_PROGRAMMING_DEADLINE_ID')) {
			$configsForProgramming['defaults']['deadlineId'] =  (int) Configuration::get('DEFAULT_PROGRAMMING_DEADLINE_ID');
		} elseif( count($programmingDeadlines) ) {
			$firstProgrammingTariff = $programmingDeadlines[0];
			$configsForProgramming['defaults']['deadlineId'] =  (int) $firstProgrammingTariff['id'];
		}

		if(Configuration::get('DEFAULT_PROGRAMMING_DISCIPLINE_ID')) {
			$configsForProgramming['defaults']['disciplineId'] =  (int) Configuration::get('DEFAULT_PROGRAMMING_DISCIPLINE_ID');
		} elseif( count($programmingDisciplineIds) ) {
			$configsForProgramming['defaults']['disciplineId'] =  (int) $programmingDisciplineIds[0];
		}

		if(Configuration::get('DEFAULT_PROGRAMMING_TASK_SIZE_ID')) {
			$configsForProgramming['defaults']['sizeId'] =  (int) Configuration::get('DEFAULT_PROGRAMMING_TASK_SIZE_ID');
		} elseif( count($configTaskSizes) ) {
			$firstTaskSize = $configTaskSizes[0];
			$configsForProgramming['defaults']['sizeId'] =  (int) $firstTaskSize['id'];
		}

		# calculations
		$calculationsTariffs = Deadline::getCalculationsDeadlines( $site->id );
		$calculationsDeadlines = array();
		foreach($calculationsTariffs as $deadline) {
			$tariff = new Deadline( (int) $deadline['deadline_id'] );
			if(Validate::isLoadedObject($tariff)) {
				$calculationsDeadlines[] = array(
					'id' => (int) $tariff->id, 
					'hours' => (int) $tariff->hrs_customer, 
					'name' => $tariff->name,
					'priceModifier' => (float) $tariff->price_modifier
				);
			}
		}

		$configsForCalculations['deadlines'] = $calculationsDeadlines;

		$calculationsDisciplines = Discipline::getCalculationsDisciplines();

		$calculationsDescriptors = array();
		foreach($calculationsDisciplines as $discipline) {
			$sql = new DbQuery();
			$sql->select('d.descriptor_id, d.discipline_id, d.task_size_id, d.example');
			$sql->from('descriptor', 'd');
			$sql->where('d.is_calculations = 1');
			$sql->where('d.discipline_id = ' . (int) $discipline['discipline_id']);
			$sql->orderBy('d.task_size_id ASC');
			$descriptors = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);

			foreach($descriptors as $descriptor) {
				$descriptor = new Descriptor( (int) $descriptor['descriptor_id'] );
				if(Validate::isLoadedObject($descriptor)) {
					$deliverables = $descriptor->getDeliverables();
					$deliverablesIds = array();
					foreach($deliverables as $deliverable) {
						$deliverablesIds[] = (int) $deliverable['deliverable_id'];
					}
				   
					$calculationsDescriptors[] = array(
						'id' => (int) $descriptor->id,
						'disciplineId' => (int) $descriptor->discipline_id,
						'taskSizeId' => (int) $descriptor->task_size_id,
						'example' => $descriptor->example,
						'deliverablesIds' => $deliverablesIds
					);
				}
			}
		}

		$configsForCalculations['descriptors'] = $calculationsDescriptors;

		$calculationsDisciplineIds = array();
		foreach($calculationsDisciplines as $discipline) {
			$calculationsDisciplineIds[] = (int) $discipline['discipline_id'];
		}

		$configsForCalculations['disciplinesIds'] = $calculationsDisciplineIds;

		$calculationsItemPricing = array();
		foreach($calculationsDisciplines as $discipline) {
			$sql = new DbQuery();
			$sql->select('ip.item_pricing_id');
			$sql->from('item_pricing', 'ip');
			$sql->where('ip.discipline_id = ' . (int) $discipline['discipline_id'] );
			$sql->orderBy('ip.task_size_id ASC');
			$itemPricing = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);

			foreach($itemPricing as $itemPrice) {
				$itemPricing = new ItemPricing( (int) $itemPrice['item_pricing_id'] );
				if(Validate::isLoadedObject($itemPricing)) {
					$calculationsItemPricing[] = array(
						'id' => (int) $itemPricing->id,
						'disciplineId' => (int) $discipline['discipline_id'],
						'sizeId' => (int) $itemPricing->task_size_id,
						'price' => (float) $itemPricing->price
					);
				}
			}
		} 

		$configsForCalculations['itemPrices'] = $calculationsItemPricing;

		$configsForCalculations['defaults'] = array(); 
		if(Configuration::get('DEFAULT_CALCULATIONS_DEADLINE_ID')) {
			$configsForCalculations['defaults']['deadlineId'] =  (int) Configuration::get('DEFAULT_CALCULATIONS_DEADLINE_ID');
		} elseif( count($calculationsDeadlines) ) {
			$firstCalculationsTariff = $calculationsDeadlines[0];
			$configsForCalculations['defaults']['deadlineId'] =  (int) $firstCalculationsTariff['id'];
		}

		if(Configuration::get('DEFAULT_CALCULATIONS_DISCIPLINE_ID')) {
			$configsForCalculations['defaults']['disciplineId'] =  (int) Configuration::get('DEFAULT_CALCULATIONS_DISCIPLINE_ID');
		} elseif( count($calculationsDisciplineIds) ) {
			$configsForCalculations['defaults']['disciplineId'] =  (int) $calculationsDisciplineIds[0];
		}

		if(Configuration::get('DEFAULT_CALCULATIONS_TASK_SIZE_ID')) {
			$configsForCalculations['defaults']['sizeId'] =  (int) Configuration::get('DEFAULT_CALCULATIONS_TASK_SIZE_ID');
		} elseif( count($configTaskSizes) ) {
			$firstTaskSize = $configTaskSizes[0];
			$configsForCalculations['defaults']['sizeId'] =  (int) $firstTaskSize['id'];
		}

		# article writing
		// Top Categories
		$getTopCategories = Discipline::getTopDisciplines( 5, Order::ARTICLE_WRITING );
		$topCategories = array();
		foreach( $getTopCategories as $discipline ) {
			$discipline = new Discipline( (int) $discipline['topic_category_id'] );
			if(Validate::isLoadedObject($discipline)) {
				$topCategories[] = (int) $discipline->id;
			}
		}

		$configsForArticleWriting['topCategoriesIds'] = $topCategories;

		// Top Paper Types
		$getArticleTopPaperTypes = PaperType::getTopPaperTypes( 5, Order::ARTICLE_WRITING );

		$topArticleContentTypes = array();
		foreach( $getArticleTopPaperTypes as $paperType ) {
			$paperType = new PaperType( (int) $paperType['paper_type_id'] );
			if(Validate::isLoadedObject($paperType)) {
				$topArticleContentTypes[] = (int) $paperType->id;
			}
		}

		$configsForArticleWriting['topContentTypes'] = $topArticleContentTypes;

		$articleWritingPaperTypes = PaperType::getPaperTypes( Order::ARTICLE_WRITING );

		//paper types
		$configArticleWritingPaperTypes = array();
		foreach ( $articleWritingPaperTypes as $paperType ) {
			$configArticleWritingPaperTypes[] = array(
				'id' => (int) $paperType['paper_type_id'],
				'title' => $paperType['title']
			);
		}

		$configsForArticleWriting['contentTypes'] = $configArticleWritingPaperTypes;

		$articleWritingTariffs = Deadline::getArticleWritingDeadlines( $site->id );
		$articleWritingDeadlines = array();
		foreach($articleWritingTariffs as $deadline) {
			$tariff = new Deadline( (int) $deadline['deadline_id'] );
			if(Validate::isLoadedObject($tariff)) {
				$articleWritingDeadlines[] = array(
					'id' => (int) $tariff->id, 
					'hours' => (int) $tariff->hrs_customer, 
					'name' => $tariff->name,
					'pricePerWord' => (float) $tariff->price_per_page
				);
			}
		}

		$configsForArticleWriting['deadlines'] = $articleWritingDeadlines;

		$articleWritingDisciplines = Discipline::getArticleWritingDisciplines();

		$articleWritingDisciplineIds = array();
		foreach($articleWritingDisciplines as $discipline) {
			$articleWritingDisciplineIds[] = (int) $discipline['discipline_id'];
		}

		$configsForArticleWriting['disciplinesIds'] = $articleWritingDisciplineIds;

		# englishTypes
		$englishTypes = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS('SELECT et.* FROM '. Db::prefix('english_type') . ' AS et ORDER BY et.title ASC');
		
		$configEnglishTypes = array();
		foreach($englishTypes as $englishType) {
			$configEnglishTypes[] = array(
				'id' => (int) $englishType['english_type_id'],
				'title' => $englishType['title']
			);
		}

		$configsForArticleWriting['englishTypes'] = $configEnglishTypes;

		#targetAudiences
		$targetAudiences = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS('SELECT ta.* FROM '. Db::prefix('target_audience') . ' AS ta ORDER BY ta.title ASC');
		
		$configTargetAudiences = array();
		foreach($targetAudiences as $targetAudience) {
			$configTargetAudiences[] = array(
				'id' => (int) $targetAudience['target_audience_id'],
				'title' => $targetAudience['title']
			);
		}

		$configsForArticleWriting['targetAudiences'] = $configTargetAudiences;

		#writingTones
		$writingTones = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS('SELECT wt.* FROM '. Db::prefix('writing_tone') . ' AS wt ORDER BY wt.title ASC');
		
		$configWritingTones = array();
		foreach($writingTones as $writingTone) {
			$configWritingTones[] = array(
				'id' => (int) $writingTone['writing_tone_id'],
				'title' => $writingTone['title'],
				'description' => $writingTone['description']
			);
		}

		$configsForArticleWriting['writingTones'] = $configWritingTones;

		#writingStyles
		$writingStyles = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS('SELECT ws.* FROM '. Db::prefix('writing_style') . ' AS ws ORDER BY ws.title ASC');

		$configWritingStyles = array();
		foreach($writingStyles as $writingStyle) {
			$configWritingStyles[] = array(
				'id' => (int) $writingStyle['writing_style_id'],
				'title' => $writingStyle['title'],
				'description' => $writingStyle['description']
			);
		}

		$configsForArticleWriting['writingStyles'] = $configWritingStyles;

		#contentFeels
		$contentFeels = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS('SELECT cf.* FROM '. Db::prefix('content_feel') . ' AS cf ORDER BY cf.title ASC');
		
		$configContentFeels = array();
		foreach($contentFeels as $contentFeel) {
			$configContentFeels[] = array(
				'id' => (int) $contentFeel['content_feel_id'],
				'title' => $contentFeel['title'],
				'description' => $contentFeel['description']
			);
		}

		$configsForArticleWriting['contentFeels'] = $configContentFeels;

		$configsForArticleWriting['defaults'] = array();
		if(Configuration::get('DEFAULT_ARTICLE_WRITING_DEADLINE_ID')) {
			$configsForArticleWriting['defaults']['deadlineId'] =  (int) Configuration::get('DEFAULT_ARTICLE_WRITING_DEADLINE_ID');
		} elseif( count($articleWritingTariffs) ) {
			$firstArticleWritingTariff = $articleWritingTariffs[0];
			$configsForArticleWriting['defaults']['deadlineId'] =  (int) $firstArticleWritingTariff['deadline_id'];
		} else {
			$configsForArticleWriting['defaults']['deadlineId'] =  null;
		}

		if(Configuration::get('DEFAULT_ARTICLE_WRITING_DISCIPLINE_ID')) {
			$configsForArticleWriting['defaults']['disciplineId'] =  (int) Configuration::get('DEFAULT_ARTICLE_WRITING_DISCIPLINE_ID');
		} elseif( count($articleWritingDisciplines) ) {
			$firstArticleWritingDiscipline = $articleWritingDisciplines[0];
			$configsForArticleWriting['defaults']['disciplineId'] =  (int) $firstArticleWritingDiscipline['discipline_id'];
		} else {
			$configsForArticleWriting['defaults']['disciplineId'] =  null;
		}

		$googleAuthConfig = array(
			'typeId' => 'google',
			'config' => [
				'clientId' => (string) Configuration::get('GOOGLE_SIGNIN_CLIENT_ID'),
				'redirectUri' => $app->base_uri . '/api/authentication/thirdPartySignIn'
			]
		);

		$authenticationProviders[] = $googleAuthConfig;

		$appleAuthConfig = array(
			'typeId' => 'apple',
			'config' => [
				'clientId' => (string) Configuration::get('APPLE_SIGNIN_CLIENT_ID'),
				'redirectUri' => $app->base_uri . '/api/authentication/thirdPartySignIn'
			]
		);

		$authenticationProviders[] = $appleAuthConfig;

		$return['authenticationProviders'] = $authenticationProviders;

		$serviceTypes = array();
		if($isAcademicWritingEnabled) {
            $serviceTypes[Order::COMPLEX_ASSIGNMENT] = array(
                'serviceId' => Order::COMPLEX_ASSIGNMENT,
                'name' => 'Academic writing',
                'description' => 'Custom papers',
            );
        }

        if($isProgrammingEnabled) {
            $serviceTypes[Order::PROGRAMMING] = array(
                'serviceId' => Order::PROGRAMMING,
                'name' => 'Programming',
                'description' => 'Tech assignments',
            );
        }

        if($isCalculationsEnabled) {
            $serviceTypes[Order::CALCULATIONS] = array(
                'serviceId' => Order::CALCULATIONS,
                'name' => 'Calculations',
                'description' => 'Problems solving',
            );
        }

        if($isArticleWritingEnabled) {
            $serviceTypes[Order::ARTICLE_WRITING] = array(
                'serviceId' => Order::ARTICLE_WRITING,
                'name' => 'Article Writing',
                'description' => 'Custom content',
            );
        }

		$defaultOrderServiceTypeId = (int) Configuration::get('DEFAULT_SERVICE_TYPE_ID', $site->id, 37);
        if(count($serviceTypes) && !ArrayUtils::has($serviceTypes, $defaultOrderServiceTypeId)) {
            $defaultServiceType = ArrayUtils::shift($serviceTypes);
            $defaultOrderServiceTypeId = $defaultServiceType['serviceId'];
        }

		$return = array_merge($return, [
			'success' => true,
            'countries' => $countries,
			'departments' => $departments,
            "defaultPaymentMethod" => $defaultPaymentMethod,
			'defaultOrderServiceTypeId' => (int) $defaultOrderServiceTypeId,
			'configsForAcademicWriting' => $configsForAcademicWriting,
			'configsForCalculations' => $configsForCalculations,
			'configsForProgramming' => $configsForProgramming,
			'configsForArticleWriting' => $configsForArticleWriting,
		]);

        return $app->response($return);
	}

	public function getManifest() {
		$app = $this->app;
        $user = $app->user;

        $pwa_app_icon = Configuration::get('PWA_APP_ICON');
        $icon_info = explode('.', $pwa_app_icon);
        $icon_url = Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID) .'/uploads/'. $icon_info[0];
        $icon_ext = $icon_info[1];

        $manifest = [
            "name" => Configuration::get('PWA_APP_NAME'),
            "short_name" => Configuration::get('PWA_SHORT_NAME'),
            "start_url" => "/dashboard",
            "display" => "standalone",
            "scope" => "/",
            "background_color" => Configuration::get('PWA_BACKGROUND_COLOR'),
            "theme_color" => Configuration::get('PWA_THEME_COLOR'),
            "orientation" => "portrait",
            "icons" => [
                [
                    "src" => $icon_url . '-72x72.' . $icon_ext,
                    "type" => "image/png",
                    "sizes" => "72x72",
                    "purpose" => "any maskable",
                ],
                [
                    "src" => $icon_url . '-96x96.' . $icon_ext,
                    "type" => "image/png",
                    "sizes" => "96x96",
                    "purpose" => "any maskable",
                ],
                [
                    "src" => $icon_url . '-128x128.' . $icon_ext,
                    "type" => "image/png",
                    "sizes" => "128x128",
                    "purpose" => "any maskable",
                ],
                [
                    "src" => $icon_url . '-144x144.' . $icon_ext,
                    "type" => "image/png",
                    "sizes" => "144x144",
                    "purpose" => "any maskable",
                ],
                [
                    "src" => $icon_url . '-152x152.' . $icon_ext,
                    "type" => "image/png",
                    "sizes" => "152x152",
                    "purpose" => "any maskable",
                ],
                [
                    "src" => $icon_url . '-192x192.' . $icon_ext,
                    "type" => "image/png",
                    "sizes" => "192x192",
                    "purpose" => "any maskable",
                ],
                [
                    "src" => $icon_url . '-384x384.' . $icon_ext,
                    "type" => "image/png",
                    "sizes" => "384x384",
                    "purpose" => "any maskable",
                ],
                [
                    "src" => $icon_url . '-512x512.' . $icon_ext,
                    "type" => "image/png",
                    "sizes" => "512x512",
                    "purpose" => "any maskable",
                ],
            ],
        ];

        return $app->response($manifest);
	}

	public function serviceWorker() {
		$app = $this->app;
        $smarty = $app->smarty; 

        $javascript = $app->getJavascript();
        $stylesheets = $app->getStylesheets();

        $javascriptList = $stylesheetsList = array();
        foreach($stylesheets['external'] as $id => $css) {
            $stylesheetsList[] = $css['uri'];
        }

        foreach($javascript['bottom']['external'] as $id => $js) {
            $javascriptList[] = $js['uri'];
        }

        $mediaList = array_merge($stylesheetsList, $javascriptList);

        $smarty->assign([
            'mediaList' => $mediaList
        ]);
        $serviceWorkerJs = $smarty->fetch("serviceWorker.tpl");

        $response = $app->response;
        $response->header('Content-Type', 'text/javascript; charset=utf-8');
        return $response->setBody($serviceWorkerJs);
	}
	
	public function tipsForWriter() {
		$app = $this->app;

		if(!Configuration::get('LEAVE_TIP_ENABLED', PROX_ADMIN_SITE_ID, false)) {
			return $app->response([
				'success' => true,
				'tipsValues' => []
			]);
		}

		$sql = new DbQuery();
        $sql->select('*');
        $sql->from('tip_option');
        $sql->orderBy('type, value ASC');
        $result = Db::getInstance()->executeS($sql);

		$tipPresets = array();
		
        foreach($result as $tipPreset) {
            $tipPresets[] = array(
                'type' => (string) $tipPreset['type'],
                'value' => (float) $tipPreset['value'],
            );
		}
		
		$tipPresets[] = [
			'type' => 'customAmount'
		];

		return $app->response([
			'success' => true,
			'tipsValues' => $tipPresets
		]);
	}

    public function getUserCountry() {
        global $globals;
        $app = $this->app;

		$countries = ArrayUtils::get($globals, 'countries');
		$country_code = $app->container->cookie->country_code ? Tools::strtolower($app->container->cookie->country_code) : "us";
		$country = ArrayUtils::get($countries, $country_code);

		return $app->response([
			"success" => true,
			"defaultCountry" => [
				"id" => $country_code,
				"name" => ArrayUtils::get($country, 'name'),
				"code" => ArrayUtils::get($country, 'code'),
				"phoneCode" => ArrayUtils::get($country, 'phoneCode')
			]
		]);
    }

}