<?php

/**
 * @package    Proxim
 * @author     Davison Pro <davisonpro.coder@gmail.com | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.0.0
 * @since      File available since Release 1.0.0
 */

namespace Proxim\Api;

use Aws\S3\Exception\S3Exception;
use Aws\S3\S3Client;
use Db;
use Proxim\Database\DbQuery;
use Proxim\User\Customer;
use Proxim\Order\Order;
use Proxim\Order\File;
use Proxim\Mail;
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Tools;
use Proxim\Validate;
use Proxim\Route;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;
use Proxim\Crypto\Hashing;
use Proxim\MailTemplate;
use Proxim\Preference\UploadType;
use Proxim\Uploader;
use Proxim\Util\StringUtils;

class Files extends Route
{

	public function getOrderFiles($orderId)
	{
		$app = $this->app;
		$payload = $app->request->post();

		$order = new Order((int) $orderId);
		if (!Validate::isLoadedObject($order)) {
			return $app->response([
				"success" => false,
			]);
		}

		$return_files = array();

		$files = $order->getFiles();
		foreach ($files as $file) {
			if ($file->uploader_id == $order->customer_id) {
				$return_files[] = $file->present();
			} else {
				if (
					$order->status_id == Order::DELIVERED ||
					$order->status_id == Order::FINISHED
				) {
					$return_files[] = $file->present();
				}
			}
		}

		return $app->response([
			"success" => true,
			"files" => $return_files,
			"showAntiPlag" => true
		]);
	}

	public function getUploadedFiles()
	{
		$app = $this->app;
		$payload = $app->request->post();

		$filesToCheck = ArrayUtils::get($payload, 'filesToCheck');

		if (!$filesToCheck || !is_array($filesToCheck)) {
			return $app->response([
				"success" => true,
				"files" => []
			]);
		}

		$files = array();
		foreach ($filesToCheck as $fileId) {
			$file = new File((int) $fileId);
			if (Validate::isLoadedObject($file)) {
				$files[$file->id] = [
					"id" => (int) $file->id,
					"name" => $file->source,
					"size" => (int) $file->size,
					"description" => "",
					"fileType" => "order_pending",
					"uploadTypeId" => null,
					"createdAt" => (float) str_pad(strtotime($file->date_add), 13, '0', STR_PAD_RIGHT)
				];
			}
		}

		return $app->response([
			"success" => true,
			"message" => null,
			"files" => $files
		]);
	}

	public function uploadFile()
	{
		$app = $this->app;
		$payload = $app->request->post();
		$user = $app->container->user;

		global $globals;
		$uploader = new Uploader();
		$uploader->setAcceptTypes($globals['extensions']);
		$uploader->setName('files');

		$uploadedFiles = $uploader->process();

		$multiple = false;
		if (isset($uploadedFiles['tmp_name'])) {
			$uploadedFile = $uploadedFiles;
		} else {
			$multiple = true;
			$uploadedFile = $uploadedFiles[0];
		}

		if (!$uploadedFile) {
			return $app->response([
				"success" => false,
			]);
		}

		$file = new File();
		$file->uploader_id = $user->id;
		$file->file_type_id = null;
		$file->file_description = null;
		$file->name = $uploadedFile['name'];
		$file->source = $uploadedFile['source_name'];
		$file->size = (int) $uploadedFile['size'];
		$file->extension = $uploadedFile['extension'];

		$ok = $file->add(true);

		if (!$ok) {
			return $app->response([
				"success" => false,
			]);
		}

		$newName = $file->name;
		if ($file->extension && !StringUtils::endsWith($newName, $file->extension)) {
			$newName .= "." . $file->extension;
		}

		$returnFile = array(
			"additional_id" => $file->id,
			"id" => $file->id,
			"name" => $file->source,
			"newName" => $newName,
			"size" =>  $file->size,
			"file_human_size" =>  $file->size,
			"file_name" => $newName,
			"url" => "not",
			"thumbnailUrl" => null,
			"uploadTypeId" => null,
			"customDesc" => "",
			"description" => "",
			"deleteType" => "DELETE",
			"deleteUrl" => null,
			"delete_link" => null,
			"download_link"  => null,
			"errors" => []
		);

		if ($multiple) {
			return $app->response([
				"success" => true,
				"files" => array($returnFile)
			]);
		} else {
			return $app->response($returnFile);
		}
	}

	public function uploadOrderFiles($orderId)
	{
		$app = $this->app;
		$payload = $app->request->post();
		$user = $app->container->user;

		$order = new Order((int) $orderId);

		if (!Validate::isLoadedObject($order)) {
			return $app->response([
				"success" => false,
			]);
		}

		$descriptions = ArrayUtils::get($payload, 'descriptions');
		$description = $descriptions[0];

		$upload_type_ids = ArrayUtils::get($payload, 'upload_type_ids');
		$upload_type_id = $upload_type_ids[0];

		global $globals;
		$uploader = new Uploader();
		$uploader->setAcceptTypes($globals['extensions']);
		$uploader->setName('files');
		$uploader->setPrefix($order->id);
		$uploadedFile = $uploader->process()[0];

		if (!$uploadedFile) {
			return $app->response([
				"success" => false,
				"errors" => [
					"This file could not be uploaded. Please contact support."
				]
			]);
		}

		if (ArrayUtils::has($payload, 'custom_descs')) {
			$custom_desc = ArrayUtils::get($payload, 'custom_descs')[0];
			$description = $custom_desc;
		} else {
			$uploadType = new UploadType((int) $description);
			$description = $uploadType->title;
		}

		$file = new File();
		$file->uploader_id = $user->id;
		$file->order_id = $order->id;
		$file->file_type_id = $upload_type_id;
		$file->file_description = $description;
		$file->name = $uploadedFile['name'];
		$file->source = $uploadedFile['source_name'];
		$file->size = $uploadedFile['size'];
		$file->extension = $uploadedFile['extension'];
		$ok = $file->add(true);

		if (!$ok) {
			return $app->response([
				"success" => false,
			]);
		}

		if ($order->is_paid) {
			$receivers = $app->getNotificationEmails();
			$writer = $order->getWriter();
        	if(Validate::isLoadedObject($writer)) {
        		$receivers[] = $writer->email;
        	}

			try {
				foreach ($receivers as $receiver) {
					Mail::Send(
						MailTemplate::TEMPLATE_ORDER_FILE_UPLOAD,
						'New file uploaded for #' . $order->id . ' at ' . Configuration::get('SITE_NAME', PROX_ADMIN_SITE_ID),
						array(
							'file_id' => $file->id,
							'file_name' => $file->name,
							'file_size' => formatBytes($file->size),
							'order_id' => $order->id,
							'order_url' => Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID) . '/order/' . $order->id
						),
						$receiver,
						null,
						null,
						null,
						null,
						null,
						null,
						null,
						null,
						null,
						null,
						PROX_ADMIN_SITE_ID
					);
				}
			} catch (\Exception $e) {
			}
		}

		$returnFile = array(
			"id" => $file->id,
			"name" => $file->source,
			"size" =>  $file->size,
			"url" => $file->getDownloadLink(),
			"thumbnailUrl" => null,
			"deleteUrl" => null,
			"deleteType" => "DELETE",
			"newName" => $file->name,
			"description" => $file->file_description
		);

		return $app->response([
			"success" => true,
			"files" => array($returnFile)
		]);
	}

	public function updateFileDescription()
	{
		$app = $this->app;
		$payload = $app->request->post();

		$upload_type_ids = ArrayUtils::get($payload, 'upload_type_ids');

		if (!$upload_type_ids || !is_array($upload_type_ids)) {
			return $app->response([
				"success" => true
			]);
		}

		$upload_type_other_descriptions = ArrayUtils::get($payload, 'upload_type_other_descriptions');

		foreach ($upload_type_ids as $fileId => $fileType) {
			$file = new File((int) $fileId);
			if (Validate::isLoadedObject($file)) {
				if (is_array($upload_type_other_descriptions)) {
					$description = ArrayUtils::get($upload_type_other_descriptions, $file->id);
					$file->file_description = $description;
				}
				$file->file_type_id = $fileType;
				$file->update();
			}
		}

		return $app->response([
			"success" => true
		]);
	}

	public function downloadFile($fileId)
	{
		$app = $this->app;
		$payload = $app->request->post();
		$user = $app->container->user;

		if (!Configuration::get('INTERNAL_FILE_STORAGE', PROX_ADMIN_SITE_ID) && !$user->isLogged()) {
			$app->setTemplate('404');
			$app->display();
		}

		$file = new File((int) $fileId);
		if (!Validate::isLoadedObject($file)) {
			$app->setTemplate('404');
			$app->display();
		}

		$order = new Order((int) $file->order_id);
		if (!Validate::isLoadedObject($order)) {
			$app->setTemplate('404');
			$app->display();
		}

		if ($user->id == $order->customer_id) {
			// update file download
			$file->downloaded = 1;
			$file->downloaded_at = DateUtils::now();
			$file->update();
		}

		$this->file_download($file);
	}

	public function file_download($file)
	{
		$app = $this->app;

		$file_path = PROX_DIR_UPLOADS . $file->source;

		$downloadFileName = $file->name;
		if ($file->extension && !StringUtils::endsWith($downloadFileName, $file->extension)) {
			$downloadFileName .= "." . $file->extension;
		}

		$mime_types = array(
			"htm" => "text/html",
			"exe" => "application/octet-stream",
			"zip" => "application/zip",
			"doc" => "application/msword",
			"docx" => "application/msword",
			"jpg" => "image/jpg",
			"php" => "text/plain",
			"xls" => "application/vnd.ms-excel",
			"ppt" => "application/vnd.ms-powerpoint",
			"gif" => "image/gif",
			"pdf" => "application/pdf",
			"txt" => "text/plain",
			"html" => "text/html",
			"png" => "image/png",
			"jpeg" => "image/jpg",
			"js" => "application/x-javascript"
		);

		if (isset($mime_types[$file->extension])) {
			$mime_type = $mime_types[$file->extension];
		} else {
			$mime_type = "application/force-download";
		}

		/** Amazon S3 File Download */
		if (Configuration::get('S3_ENABLED', PROX_ADMIN_SITE_ID)) {
			$s3_region = Configuration::get('S3_REGION', PROX_ADMIN_SITE_ID);
			$s3_key = Configuration::get('S3_KEY', PROX_ADMIN_SITE_ID);
			$s3_secret = Configuration::get('S3_SECRET', PROX_ADMIN_SITE_ID);
			$s3_bucket = Configuration::get('S3_BUCKET', PROX_ADMIN_SITE_ID);

			try {
				$s3Client = S3Client::factory(array(
					'version'    => 'latest',
					'region'      => $s3_region,
					'credentials' => array(
						'key'    => $s3_key,
						'secret' => $s3_secret,
					)
				));

				// Generate a signed URL that expires in a specific time (e.g., 5 minutes)
				$signedUrl = $s3Client->getObjectUrl($s3_bucket, 'uploads/' . $file->source, '+5 minutes');

				// Set the appropriate headers to force the download
				header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
				header('Pragma: no-cache');
				header('Expires: 0');
				header('Content-Disposition: attachment; filename="' . $downloadFileName . '"');
				header('Content-Type: application/octet-stream');

				// Output the file content directly to the browser
				readfile($signedUrl);
				exit;

				// Get the object.
				$result = $s3Client->getObject([
					'Bucket' => $s3_bucket,
					'Key'    => 'uploads/' . $file->source
				]);

				header('Pragma: public');
				header("Expires: 0");
				header('Connection: Keep-Alive');
				header("Cache-Control: public, must-revalidate, post-check=0, pre-check=0");
				header('Content-Type: ' . $mime_type);
				header('Content-Disposition: attachment; filename="' . $downloadFileName . '"');
				header("Content-Transfer-Encoding: binary");
				header('Accept-Ranges: bytes');
				header("Content-Type: {$result['ContentType']}");
				echo $result['Body'];
			} catch (S3Exception $e) {
				$app->setTemplate('404');
				$app->display();
			}
		} else {
			$downloadFile = @fopen($file_path, "rb");
			if (!$downloadFile) {
				$app->setTemplate('404');
				$app->display();
			}

			$downloadFileSize = filesize($file_path);

			@ob_end_clean();
			if (ini_get('zlib.output_compression')) {
				ini_set('zlib.output_compression', 'Off');
			}

			header('Pragma: public');
			header("Expires: 0");
			header('Connection: Keep-Alive');
			header("Cache-Control: public, must-revalidate, post-check=0, pre-check=0");
			header('Content-Type: ' . $mime_type);
			header('Content-Disposition: attachment; filename="' . $downloadFileName . '"');
			header("Content-Transfer-Encoding: binary");
			header('Accept-Ranges: bytes');

			if (isset($_SERVER['HTTP_RANGE'])) {
				list($a, $range) = explode("=", $_SERVER['HTTP_RANGE'], 2);
				list($range) = explode(",", $range, 2);
				list($range, $range_end) = explode("-", $range);
				$range = intval($range);
				if (!$range_end) {
					$range_end = $size - 1;
				} else {
					$range_end = intval($range_end);
				}

				$new_length = $range_end - $range + 1;
				header("HTTP/1.1 206 Partial Content");
				header("Content-Length: $new_length");
				header("Content-Range: bytes $range-$range_end/$downloadFileSize");
			} else {
				$new_length = $downloadFileSize;
				header("Content-Length: " . $downloadFileSize);
			}

			$chunksize = 1 * (1024 * 1024);
			$bytes_sent = 0;

			if (isset($_SERVER['HTTP_RANGE'])) {
				fseek($downloadFile, $range);
			}

			while (
				!feof($downloadFile) &&
				(!connection_aborted()) &&
				($bytes_sent < $new_length)
			) {
				$buffer = fread($downloadFile, $chunksize);
				echo ($buffer);
				flush();
				$bytes_sent += strlen($buffer);
			}

			@fclose($downloadFile);
			exit;
		}
	}

	public function deleteFile($fileId)
	{
		$app = $this->app;
		$payload = $app->request->post();
		$user = $app->container->user;

		$file = new File((int) $fileId);
		if (!Validate::isLoadedObject($file)) {
			return $app->response([
				"success" => false
			]);
		}

		$order = new Order($file->order_id);
		if (!Validate::isLoadedObject($order) || $order->customer_id != $user->id) {
			return $app->response([
				"success" => false
			]);
		}

		$orderPayed = $order->is_paid || $order->status_id = Order::FREE_INQUIRY ? true : false;

		$receivers = $app->getNotificationEmails();

		try {
			foreach ($receivers as $receiver) {
				Mail::Send(
					MailTemplate::TEMPLATE_ORDER_FILE_DELETE,
					'File delete request for #' . $order->id . ' at ' . Configuration::get('SITE_NAME'),
					array(
						'file_id' => $file->id,
						'file_name' => $file->name,
						'file_size' => formatBytes($file->size),
						'order_id' => $order->id,
						'order_url' => Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID) . '/order/' . $order->id
					),
					$receiver,
					null,
					null,
					null,
					null,
					null,
					null,
					null,
					null,
					null,
					null,
					PROX_ADMIN_SITE_ID
				);
			}
		} catch (\Exception $e) {
		}

		return $app->response([
			"orderPayed" => (bool) $orderPayed
		]);
	}

	public function cloudStorage()
	{
		$s3_region = Configuration::get('S3_REGION', PROX_ADMIN_SITE_ID);
		$s3_key = Configuration::get('S3_KEY', PROX_ADMIN_SITE_ID);
		$s3_secret = Configuration::get('S3_SECRET', PROX_ADMIN_SITE_ID);
		$s3_bucket = Configuration::get('S3_BUCKET', PROX_ADMIN_SITE_ID);

		$s3Client = S3Client::factory(array(
			'version'    => 'latest',
			'region'      => $s3_region,
			'credentials' => array(
				'key'    => $s3_key,
				'secret' => $s3_secret,
			)
		));

		try {
			$status = $s3Client->uploadDirectory(PROX_DIR_ROOT . 'uploads', $s3_bucket . '/uploads');
			print_r($status);
		} catch (Aws\Exception\S3Exception $e) {
			echo "There was an error uploading the directory";
		}
	}
}
