<?php 
/**
 * @package    Proxim
 * @author     Davison Pro <davis@davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.5.0
 * @since      File available since Release 1.0.0
 */

use Proxim\Database\DbQuery;
use Proxim\Cache\Cache;
use Proxim\ObjectModel;
use Proxim\Validate;
use Proxim\Tools;
use Proxim\Util\DateUtils;
use Proxim\Crypto\Hashing;

/**
 * Affiliate
 */
class Affiliate extends ObjectModel {

    const START_ID = 1000;

    const TYPE_PERCENT = 'percent';

    const TYPE_AMOUNT = 'amount';

    /** @var $id Affiliate ID */
    public $id;

    /** @var string $name */
    public $name;

    /** @var string $email */
    public $email;

    /** @var string $phone */
    public $phone;

    /** @var string affiliate_code */
    public $affiliate_code;

    /** @var string affiliate_type */
    public $affiliate_type = self::TYPE_PERCENT;

    /** @var int affiliate_value */
    public $affiliate_value = 0;

    /** @var string $password */
    public $password;

    /** @var string $reset_password_token */
    public $reset_password_token;

    /** @var string $reset_password_validity */
    public $reset_password_validity;

    /** @var string $last_passwd_gen */
    public $last_passwd_gen;

    /** @var string $secure_key */
    public $secure_key;

    /** @var float total_earnings */
    public $total_earnings = 0;

    /** @var float total_paid */
    public $total_paid = 0;

    /** @var float total_refunded */
    public $total_refunded = 0;

    /** @var float balance */
    public $balance = 0;

    /** @var int visits */
    public $visits = 0;

    /** @var bool is_banned */
    public $is_banned = 0;

    /** @var string $date_upd */
    public $date_upd;

    /** @var string $date_add */
    public $date_add;

    /** @var bool is the user logged in */
    public $logged = false;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'affiliate',
        'primary' => 'affiliate_id',
        'fields' => array(
            'name' => array('type' => self::TYPE_STRING, 'validate' => 'isName', 'required' => false, 'size' => 255),
            'email' => array('type' => self::TYPE_STRING, 'validate' => 'isEmail', 'required' => true, 'size' => 255),
            'phone' => array('type' => self::TYPE_STRING, 'validate' => 'isPhoneNumber', 'required' => false, 'size' => 50),
            'affiliate_code' => array('type' => self::TYPE_STRING, 'validate' => 'isGenericName', 'required' => true),
            'affiliate_type' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'affiliate_value' => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'password' => array('type' => self::TYPE_STRING, 'validate' => 'isPasswd', 'required' => true, 'size' => 255),
            'reset_password_token' => array('type' => self::TYPE_STRING, 'validate' => 'isSha1', 'size' => 40),
            'reset_password_validity' => array('type' => self::TYPE_DATE, 'validate' => 'isDateOrNull'),
            'last_passwd_gen' => array('type' => self::TYPE_DATE, 'validate' => 'isDateOrNull'),
            'secure_key' => array('type' => self::TYPE_STRING, 'validate' => 'isMd5'),
            'total_earnings' => array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'total_paid' => array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'total_refunded' => array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'balance' => array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'visits' => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
			'is_banned' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
			'date_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDateOrNull'),
			'date_add' => array('type' => self::TYPE_DATE, 'validate' => 'isDateOrNull'),
        )
	);

    /**
     * constructor.
     *
     * @param null $id
     */
    public function __construct($id = null)
    {
        parent::__construct($id);
    }

    public function add($autoDate = true, $nullValues = true)
    {
        $this->secure_key = md5(uniqid(rand(), true));
        $this->last_passwd_gen = DateUtils::now();

        if ( !$this->getNextAffiliateId() ) {
            $this->id = self::START_ID;
        } else {
            $next_affiliate_id = Db::getInstance()->getValue('SELECT MAX(`affiliate_id`)+2 FROM `' . DB_PREFIX . 'affiliate`');
            $this->id = $next_affiliate_id;
        }

        $this->force_id = true;

        $success = parent::add($autoDate, $nullValues);

        return $success;
    }

    /**
     * Return user instance from its e-mail (optionally check password).
     *
     * @param string $email e-mail
     * @param string $plaintextPassword Password is also checked if specified
     *
     * @return bool|User User instance
     */
    public function getByEmail($email, $plaintextPassword = null)
    {
        if (!Validate::isEmail($email) || ($plaintextPassword && !Validate::isPasswd($plaintextPassword))) {
            return false;
        }
        
        $sql = new DbQuery();
        $sql->select('a.*');
        $sql->from('affiliate', 'a');
        $sql->where('a.`email` = \'' . pSQL($email) . '\'');
        
        $result = Db::getInstance()->getRow($sql);
        if (!$result) {
            return false;
        }

        $crypto = new Hashing();

        $passwordHash = $result['password'];
        $shouldCheckPassword = !is_null($plaintextPassword);
        if ($shouldCheckPassword && !$crypto->checkHash($plaintextPassword, $passwordHash)) {
            return false;
        }
        
        $this->id = $result['affiliate_id'];
        foreach ($result as $key => $value) {
            if (property_exists($this, $key)) {
                $this->{$key} = $value;
            }
        }
        
        if ($shouldCheckPassword && !$crypto->isFirstHash($plaintextPassword, $passwordHash)) {
            $this->password = $crypto->hash($plaintextPassword);
            $this->update();
        }

        return $this;

    }

    /**
     * Check if affiliate password is the right one.
     *
     * @param int $affiliate_id Affiliate ID
     * @param string $passwordHash Hashed password
     *
     * @return bool result
     */
    public static function checkPassword($affiliate_id, $passwordHash)
    {
        if (!Validate::isUnsignedId($affiliate_id)) {
            die(Tools::displayError());
        }

        $cacheId = 'Affiliate::checkPassword' . (int) $affiliate_id . '-' . $passwordHash;
        if (!Cache::isStored($cacheId)) {
            $sql = new DbQuery();
            $sql->select('a.`affiliate_id`');
            $sql->from('affiliate', 'a');
            $sql->where('a.`affiliate_id` = ' . (int) $affiliate_id);
            $sql->where('a.`password` = \'' . pSQL($passwordHash) . '\'');

            $result = (bool) Db::getInstance(PROX_USE_SQL_SLAVE)->getValue($sql);

            Cache::store($cacheId, $result);

            return $result;
        }

        return Cache::retrieve($cacheId);
    }

    /**
     * Check if a Affiliate Code already exists.
     *
     * @return bool Indicates whether the Code already exists
     */
    public static function affiliateCodeExists($affiliate_code)
    {
        $cacheId = 'Affiliate::affiliateCodeExists' . pSQL($affiliate_code);
        if (!Cache::isStored($cacheId)) {
            $sql = new DbQuery();
            $sql->from('affiliate');
            $sql->where('affiliate_code = \'' . pSQL($affiliate_code) . '\'');
            $result = (int) Db::getInstance(PROX_USE_SQL_SLAVE)->getValue($sql);
            Cache::store($cacheId, $result);

            return $result;
        }

        return Cache::retrieve($cacheId);
    }

    /**
     * Check user information and return user validity.
     *
     * @return bool user validity
     */
    public function isLogged()
    {
        /* Customer is valid only if it can be load and if session is active */
        return $this->logged == 1 && (int) $this->id && Affiliate::checkPassword($this->id, $this->password);
    }

    /**
     * Fill Reset password unique token with random sha1 and its validity date. For forgot password feature.
     */
    public function stampResetPasswordToken()
    {
        $salt = $this->id . '-' . $this->secure_key;
        $this->reset_password_token = sha1(time() . $salt);
        $this->reset_password_validity = date('Y-m-d H:i:s', strtotime('+' . 1440 . ' minutes'));
    }

    /**
     * Test if a reset password token is present and is recent enough to avoid creating a new one (in case of customer triggering the forgot password link too often).
     */
    public function hasRecentResetPasswordToken()
    {
        if (!$this->reset_password_token || $this->reset_password_token == '') {
            return false;
        }

        // TODO maybe use another 'recent' value for this test. For instance, equals password validity value.
        if (!$this->reset_password_validity || strtotime($this->reset_password_validity) < time()) {
            return false;
        }

        return true;
    }
    
    /**
     * Returns the valid reset password token if it validity date is > now().
     */
    public function getValidResetPasswordToken()
    {
        if (!$this->reset_password_token || $this->reset_password_token == '') {
            return false;
        }

        if (!$this->reset_password_validity || strtotime($this->reset_password_validity) < time()) {
            return false;
        }

        return $this->reset_password_token;
    }

    /**
     * Delete reset password token data.
     */
    public function removeResetPasswordToken()
    {
        $this->reset_password_token = null;
        $this->reset_password_validity = null;
    }

    /**
     * This method return the ID of the next affiliate.
     *
     * @since 1.0.0
     *
     * @return int
     */
    public function getNextAffiliateId()
    {
        return Db::getInstance()->getValue('
            SELECT affiliate_id
            FROM ' . DB_PREFIX . 'affiliate
            WHERE affiliate_id > ' . (int) $this->id . ' 
            ORDER BY affiliate_id ASC');
    }
}