<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Currency;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Order\AdditionalService;
use Proxim\Order\Order;
use Proxim\Site\Site;
use Proxim\User\Customer;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

require_once dirname(__FILE__) . '/vendor/autoload.php';
require_once dirname(__FILE__) . '/classes/AuthorizeNetPayment.php';

class Authorize extends PaymentModule
{
    /**
    * Authorize configuration
    */
    const AUTHORIZE_ENABLED = 'AUTHORIZE_ENABLED';
    const AUTHORIZE_MODE = 'AUTHORIZE_MODE';
    const AUTHORIZE_API_LOGIN_ID = 'AUTHORIZE_API_LOGIN_ID';
    const AUTHORIZE_TRANSACTION_KEY = 'AUTHORIZE_TRANSACTION_KEY';

    public function __construct()
    {
        $this->name = 'authorize';
        $this->icon = 'fa fa-credit-card';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Authorize.net';
        $this->description = 'Set up Authorize.net as a payment method and enable users to pay without leaving your website';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::AUTHORIZE_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function processOrderCheckout($order, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;

        $smarty->assign([ 
            'currency' => 'USD',
            'amount' => toFixed($order->total),
            'redirectUri' => $app->base_uri . '/dashboard/order/' . $order->id,
            'base_uri' => $app->base_uri,
            'css_path' => $this->local_path . 'views/_css.authorize.tpl'
        ]);
        
        return $this->getTemplateContent('authorize.order');
    }

    public function processPaperCheckout($paper, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;

        $smarty->assign([
            'currency' => 'USD',
            'amount' => toFixed($paper->price),
            'redirectUri' => $app->base_uri . '/dashboard/paper/' . $paper->id,
            'base_uri' => $app->base_uri,
            'css_path' => $this->local_path . 'views/_css.authorize.tpl'
        ]);
        
        return $this->getTemplateContent('authorize.paper');
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;

        $smarty->assign([
            'currency' => 'USD',
            'amount' => toFixed($addPayment->total),
            'redirectUri' => $app->base_uri . '/dashboard/add-payment/' . $addPayment->id,
            'base_uri' => $app->base_uri,
            'css_path' => $this->local_path . 'views/_css.authorize.tpl'
        ]);
        
        return $this->getTemplateContent('authorize.add-payment');
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;

        $smarty->assign([
            'currency' => 'USD',
            'amount' => toFixed($amount),
            'redirectUri' => $app->base_uri . '/dashboard/wallet',
            'base_uri' => $app->base_uri,
            'css_path' => $this->local_path . 'views/_css.authorize.tpl'
        ]);

        return $this->getTemplateContent('authorize.wallet');
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;

        $smarty->assign([
            'currency' => 'USD',
            'amount' => toFixed($amount),
            'redirectUri' => $app->base_uri . '/dashboard/order/' . $order_id,
            'base_uri' => $app->base_uri,
            'css_path' => $this->local_path . 'views/_css.authorize.tpl'
        ]);

        return $this->getTemplateContent('authorize.tip');
    } 

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPaymentAfter($params) 
    { 
        $app = $this->application;
        $smarty = $this->smarty;

        if(!ArrayUtils::has($params, 'payNow')) {
            $app->setTemplate('404');
            $app->display();
        }

        $handle = ArrayUtils::get($params, 'handle');
        $nodeId = (int) ArrayUtils::get($params, 'nodeId');

        $authorizeNetPayment = new AuthorizeNetPayment(
            Configuration::get(self::AUTHORIZE_API_LOGIN_ID),
            Configuration::get(self::AUTHORIZE_TRANSACTION_KEY),
            Configuration::get(self::AUTHORIZE_MODE),
            $nodeId
        );

        $response = $authorizeNetPayment->chargeCreditCard($params);

        if ($response != null) {
            // Check to see if the API request was successfully received and acted upon
            if ($response->getMessages()->getResultCode() == "Ok") {
                // Since the API request was successful, look for a transaction response
                // and parse it to display the results of authorizing the card
                $trxResponse = $response->getTransactionResponse();

                if ($trxResponse != null && $trxResponse->getMessages() != null) {
                    $params['paymentId'] = $trxResponse->getTransId();
                    PaymentModule::processPayment( $nodeId, $handle, $params );
                    $smarty->assign(array(
                        'status' => 'success'
                    ));
                } else {
                    $smarty->assign(array(
                        'status' => 'error'
                    ));
                }
                // Or, print errors if the API request wasn't successful
            } else {
                $smarty->assign(array(
                    'status' => 'error'
                ));
            }      
        } else {
            $smarty->assign(array(
                'status' => 'error'
            ));
        }
    }

}