<?php
/**
 * @package    Proxim
 * @author     Davison Pro <davis@davisonpro.dev | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.5.0
 * @since      File available since Release 1.0.0
 */

use Proxim\Application;
use Proxim\Configuration;
use Proxim\Coupon;
use Proxim\Crypto\Hashing;
use Proxim\Currency;
use Proxim\Database\DbQuery;
use Proxim\Mail;
use Proxim\MailTemplate;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Order\File;
use Proxim\Order\Order;
use Proxim\Preference\Deadline;
use Proxim\Tools;
use Proxim\User\Customer;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;
use Proxim\Validate;

class Chatbot extends Module
{
    public function __construct()
    {
        $this->name = 'chatbot';
        $this->icon = 'fa fa-chat';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Chatbot';
        $this->description = 'ChatBot that can help Increase your store Sales perceptibly. Shoppers can converse fluidly with the Bot.';

        $this->bootstrap = true;
        parent::__construct();
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function hookActionBeforeInitAffiliate() {
        global $globals;

        $app = $this->application;
        $user = $app->container->user;
        $payload = $app->request->get();

        if(ArrayUtils::has($payload, 'chatbot')) {
            $sql = new DbQuery();
            $sql->select('d.*');
            $sql->from('deadline', 'd');
            $sql->where('d.is_academic_writing = 1' );
            $sql->where('d.academic_level_id = ' . (int) LEVEL_UNIVERSITY );
            $sql->where('d.site_id = ' . (int) PROX_SITE_ID );
            $sql->orderBy('d.deadline_id DESC');
            $deadlines = Db::getInstance()->executeS($sql);

            $tarrifsIds = array();
            foreach($deadlines as $deadline) {
                $tarrifsIds[] = array(
                    'id' => (int) $deadline['deadline_id'],
                    'name' => $deadline['name']
                ); 
            }
            
            $isUserAuthorised = $user->isLogged();

            $this->smarty->assign([
                'base_uri' => $app->base_uri,
                'isUserAuthorised' => $isUserAuthorised,
                'chatBotOrderUrl' => $app->base_uri . '/order',
                'tarrifsIds' => $tarrifsIds,
            ]);

            $this->showTemplate('chatbot');
            exit;
        }

        if(ArrayUtils::has($payload, 'is_chatbot')) {
            $email = ArrayUtils::get($payload, 'email');
            if(!$user->isLogged() && Validate::isEmail($email)) {

                $user = $user->getByEmail($email);
                if(!Validate::isLoadedObject($user)) {
                    $password = Tools::randomGen(8);
        
                    $user = new Customer();
                    $user->google_analytics_id = Tools::randomGen(32);
                    $user->email = $email;
        
                    $crypto = new Hashing();
                    $user->password = $crypto->hash(
                        $password,
                        PROX_COOKIE_KEY
                    );
                    $user->is_subscribed = 1;
                    $user->policy_accepted = 1;
                    $user->policy_accepted_at =  DateUtils::now();
                    $user->last_activity = DateUtils::now();
                    $user->reg_date = DateUtils::now();
                    $user->add(true, true);
        
            
                    $mailParams = array(
                        'email' => $user->email,
                        'password' => $password,
                        'my_account_url' => Configuration::get('SITE_DOMAIN', $user->site_id) . '/dashboard',
                    );
        
                    Mail::Send(
                        MailTemplate::TEMPLATE_CUSTOMER_REGISTRATION,
                        'Welcome! Please log into your personal order page',
                        $mailParams,
                        $user->email
                    );
                }

                // update user 
		        $app->updateUser( $user );
            }
    
            $title = ArrayUtils::get($payload, 'title');
            $pages = (int) ArrayUtils::get($payload, 'pages');
            $paperDetails = ArrayUtils::get($payload, 'paperDetails');
            $writerCategoryId = (int) ArrayUtils::get($payload, 'writerCategoryId');
            $paperDetails = ArrayUtils::get($payload, 'paperDetails');
            $deadlineId = ArrayUtils::get($payload, 'deadlineId');

            $additionalMaterialsIds = ArrayUtils::get($payload, 'additionalMaterialsIds');
            $additionalMaterialsIds = explode('-', $additionalMaterialsIds);

            $writerPreferences = ArrayUtils::get($globals, 'writerPreferences');

            $writerPercent = 0;
            if (ArrayUtils::has($writerPreferences, $writerCategoryId)) {
                $writerPreference = ArrayUtils::get($writerPreferences, $writerCategoryId);
                $writerPercent = ArrayUtils::has($writerPreference, 'percent') ? (int) ArrayUtils::get($writerPreference, 'percent') : 0;
            }

            $order = new Order();
            $order->customer_id = $user->id;
            $order->service_type = Order::COMPLEX_ASSIGNMENT;
            $order->title = $title;
            $order->academic_level_id = LEVEL_UNIVERSITY;
            $order->pages = $pages;
            $order->paper_details = $paperDetails;

            // payment method
            $paymentModules = PaymentModule::getInstalledPaymentModules();
            $paymentMethods = array();
            foreach($paymentModules as $paymentModule) {
                $paymentModule = Module::getInstanceByName($paymentModule['name']);
                if(Validate::isLoadedObject($paymentModule)) {
                    if($paymentModule->isActive()) {
                        $paymentMethods[] = $paymentModule->name;
                    }
                }
            }

            $defaultPaymentMethod = Configuration::get('PAYMENT_METHOD_DEFAULT');
            if(!$defaultPaymentMethod && count($paymentMethods)) {
                $defaultPaymentMethod = $paymentMethods[0];
            }

            $order->payment_method = $defaultPaymentMethod;
            
            // currency
            $currency = new Currency( Configuration::get('CURRENCY_DEFAULT') );
            if(Validate::isLoadedObject($currency)) {
                $order->currency_id = $currency->id;
                $order->currency_to_usd_rate = $currency->conversion_rate;
            }

            $order->writer_category_id = $writerCategoryId;
            $order->writer_percent = $writerPercent;
		    
            $tariff = new Deadline( (int) $deadlineId );
            if(!Validate::isLoadedObject($tariff)) {
                $deadlines = Deadline::getAcademicDeadlines( $user->site_id );

                $tarrifs = array();
                foreach($deadlines as $index => $columns) {
                    foreach($columns as $key => $value) {
                        if ($key == 'hrs_customer' && ($value >= '12' || $value <= '48') ) {
                            $tarrifs[] = $columns;
                        }
                    }
                }

                $tariff = new Deadline( $tarrifs[0]['deadline_id'] );
            }
            
            $order->tariff_id = $tariff->id;
            $order->price_per_page = (float) $tariff->price_per_page;
            $order->hrs_customer = (int) $tariff->hrs_customer;
            $order->hrs_writer = (int) $tariff->hrs_writer;
            $order->customer_deadline = DateUtils::date(strtotime('+'. $tariff->hrs_customer . ' hours'));

            $order->paper_type_id = 64;
            $order->paper_type_option = 'Check Instructions';
            $order->topic_category_id = 52;
            $order->topic_category_option = 'Check Instructions';

            $calc = $order->calculate();
            $baseCost = ArrayUtils::get($calc, 'rawCost');
            $order->raw_cost = $baseCost;
            $order->total = ArrayUtils::get($calc, 'totalCost');
            $order->writer_pay = ArrayUtils::get($calc, 'writerCost');
            $order->editor_pay = ArrayUtils::get($calc, 'editorCost');
			$order->status_id = Order::FAKE;
            $order->add();

            if(is_array($additionalMaterialsIds) && !empty($additionalMaterialsIds)) {
                foreach ($additionalMaterialsIds as $additionalFileId) {
					$orderFile = new File( (int) $additionalFileId );
					// insert if the file exists
					if ( Validate::isLoadedObject($orderFile) ) {
                        $orderFile->order_id = $order->id;
                        $orderFile->customer_id = $order->customer_id;
                        $orderFile->update();
                    }
                }
            }

            $redirectUri = Configuration::get('SITE_DOMAIN') . '/checkout/' . $order->payment_method . '/order/' . $order->id;

            Tools::redirectLink($redirectUri);
            exit;
        }
        
    }

}