<?php 
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\PaymentModule;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;

class CyberSource extends PaymentModule {

    // CyberSource Standard Transaction Endpoints
	public $test_url = "https://testsecureacceptance.cybersource.com/silent/pay";
	public $live_url = "https://secureacceptance.cybersource.com/silent/pay";

    /**
    * CyberSource configuration
    */
    const CYBERSOURCE_ENABLED = 'CYBERSOURCE_ENABLED';
    const CYBERSOURCE_MODE = 'CYBERSOURCE_MODE';
    const CYBERSOURCE_MERCHANT_ID = 'CYBERSOURCE_MERCHANT_ID';
    const CYBERSOURCE_TRANSACTION_KEY = 'CYBERSOURCE_TRANSACTION_KEY';
    const CYBERSOURCE_SECRET_KEY = 'CYBERSOURCE_SECRET_KEY';

    public function __construct()
    {
        $this->name = 'cybersource';
        $this->icon = 'fa fa-credit-card';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'CyberSource';
        $this->description = 'Set up CyberSource as a payment method and enable users to pay without leaving your website';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::CYBERSOURCE_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    public function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function getActionUrl() {
        $api_mode = Configuration::get(self::CYBERSOURCE_MODE);
        return $api_mode == 'live' ? $this->live_url : $this->test_url;
    }

    public function makeSignature($request_form_data) {
        $secret_key = Configuration::get(self::CYBERSOURCE_SECRET_KEY);
        return $this->signData($this->buildDataToSign($request_form_data), $secret_key);
    }

    public function signData($data, $secret_key)
    {
        return base64_encode(hash_hmac('sha256', $data, $secret_key, true));
    }

    public function buildDataToSign($transaction)
    {
        $signedFieldNames = explode(",", $transaction["signed_field_names"]);
        foreach ($signedFieldNames as $field) {
            $dataToSign[] = $field . "=" . $transaction[$field];
        }

        return $this->commaSeparate($dataToSign);
    }

    public function commaSeparate($dataToSign)
    {
        return implode(",", $dataToSign);
    }

    public function getCountries() {
        $countries = array(
            "AU" => "Australia",
            "CA" => "Canada",
            "CH" => "China",
            "DE" => "Germany",
            "JP" => "Japan",
            "PH" => "Philippines",
            "ZA" => "South Africa",
            "AE" => "United Arab Emirates",
            "GB" => "United Kingdom",
            "US" => "United States"
        );

        return $countries;
    }

    public function setVariables() {
        $app = $this->application;
        $smarty = $this->smarty;

        $us_states = array(
            'AL'=>"Alabama",  
            'AK'=>"Alaska",  
            'AZ'=>"Arizona",  
            'AR'=>"Arkansas",  
            'CA'=>"California",  
            'CO'=>"Colorado",  
            'CT'=>"Connecticut",  
            'DE'=>"Delaware",  
            'DC'=>"District Of Columbia",  
            'FL'=>"Florida",  
            'GA'=>"Georgia",  
            'HI'=>"Hawaii",  
            'ID'=>"Idaho",  
            'IL'=>"Illinois",  
            'IN'=>"Indiana",  
            'IA'=>"Iowa",  
            'KS'=>"Kansas",  
            'KY'=>"Kentucky",  
            'LA'=>"Louisiana",  
            'ME'=>"Maine",  
            'MD'=>"Maryland",  
            'MA'=>"Massachusetts",  
            'MI'=>"Michigan",  
            'MN'=>"Minnesota",  
            'MS'=>"Mississippi",  
            'MO'=>"Missouri",  
            'MT'=>"Montana",
            'NE'=>"Nebraska",
            'NV'=>"Nevada",
            'NH'=>"New Hampshire",
            'NJ'=>"New Jersey",
            'NM'=>"New Mexico",
            'NY'=>"New York",
            'NC'=>"North Carolina",
            'ND'=>"North Dakota",
            'OH'=>"Ohio",  
            'OK'=>"Oklahoma",  
            'OR'=>"Oregon",  
            'PA'=>"Pennsylvania",  
            'RI'=>"Rhode Island",  
            'SC'=>"South Carolina",  
            'SD'=>"South Dakota",
            'TN'=>"Tennessee",  
            'TX'=>"Texas",  
            'UT'=>"Utah",  
            'VT'=>"Vermont",  
            'VA'=>"Virginia",  
            'WA'=>"Washington",  
            'WV'=>"West Virginia",  
            'WI'=>"Wisconsin",  
            'WY'=>"Wyoming",
            "BC" => "British Columbia", 
            "ON" => "Ontario", 
            "NL" => "Newfoundland and Labrador", 
            "NS" => "Nova Scotia", 
            "PE" => "Prince Edward Island", 
            "NB" => "New Brunswick", 
            "QC" => "Quebec", 
            "MB" => "Manitoba", 
            "SK" => "Saskatchewan", 
            "AB" => "Alberta", 
            "NT" => "Northwest Territories", 
            "NU" => "Nunavut",
            "YT" => "Yukon Territory"
        );

        $card_types = array(
			'001' => 'Visa',
			'002' => 'MasterCard',
			'003' => 'American Express',
			'042' => 'Maestro Int\'l'
		);

        $smarty->assign([ 
            'card_types' => $card_types,
            'us_states' => $us_states,
            'countries' => $this->getCountries(),
        ]);
    }

    public function processOrderCheckout($order, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;
        $payload = $app->container->request->post();

        $merchant_id = Configuration::get(self::CYBERSOURCE_MERCHANT_ID);
        $access_key = Configuration::get(self::CYBERSOURCE_TRANSACTION_KEY);
        $_form = ArrayUtils::has($payload, 'confirm_pay') ? 'signed': 'unsigned';

        $this->setVariables();

        if($_form == 'signed') {
            $transaction = [
                'bill_to_forename' => ArrayUtils::get($payload, 'bill_to_forename'),
                'bill_to_surname' => ArrayUtils::get($payload, 'bill_to_surname'),
                'bill_to_address_line1' => ArrayUtils::get($payload, 'bill_to_address_line1'),
                'bill_to_address_country' => ArrayUtils::get($payload, 'bill_to_address_country'),
                'bill_to_address_state' => ArrayUtils::get($payload, 'bill_to_address_state'),
                'bill_to_address_city' => ArrayUtils::get($payload, 'bill_to_address_city'),
                'bill_to_address_postal_code' => ArrayUtils::get($payload, 'bill_to_address_postal_code'),
                'access_key' => $access_key,
                'profile_id' => $merchant_id,
                'transaction_uuid' => Tools::randomGen(10),
                "unsigned_field_names" => 'card_number,card_expiry_date,card_type',
                'signed_date_time' => gmdate("Y-m-d\TH:i:s\Z"),
                'locale' => 'en',
                'transaction_type' => 'sale',
                "reference_number" => 'order_' . $order->id,
                'currency' => $currency->iso_code,
                'amount' => toFixed($order->total),
                'payment_method' => 'card',
                'bill_to_email' => $user->email,
                'bill_to_phone' => $user->phone
            ];

            $signed_field_names = "signed_field_names,";
            $signed_field_names .= implode( ',', array_keys($transaction) );
    
            $transaction['signed_field_names'] = $signed_field_names;
            $transaction['signature'] = $this->makeSignature($transaction);

            $smarty->assign([ 
                'action_url' => $this->getActionUrl(),
                'transaction' => $transaction,
            ]);
        } else {
            $smarty->assign([ 
                'action_url' => $app->base_uri . '/checkout/' . $this->name . '/order/' . $order->id
            ]);
        }

        $smarty->assign([
            '_form' => $_form,
            'redirectUri' => $app->base_uri . '/dashboard/order/' . $order->id,
            'base_uri' => $app->base_uri,
            'css_path' => $this->local_path . 'views/_css.cybersource.tpl'
        ]);
        
        return $this->getTemplateContent('cybersource.order');
    }
    
    public function getCcType( $cardnum ) { 
	  		
        /* Visa */
        if ( preg_match( "/^4(\d{12}|\d{15})$/", $cardnum ) ) { 
            $type = '001'; 
        /* MasterCard */	
        } else if( preg_match( "/^5[1-5]\d{14}$/", $cardnum ) ) { 
            $type = '002'; 
        /* American Express */	
        } else if( preg_match( "/^3[47]\d{13}$/", $cardnum ) ) { 
            $type = '003';
        /* Discover */
        } else if( preg_match( "/^6011\d{12}$/", $cardnum ) ){ 
            $type = '004'; // Discover
        /* Diners Club */
        } else if( preg_match( "/^[300-305]\d{11}$/", $cardnum ) || preg_match( "/^3[68]\d{12}$/", $cardnum ) ){ 
            $type = '005'; 
        /* EnRoute */
        } else if ( preg_match( "/^2(014|149)\d{11}$/", $cardnum) ) {
            $type = '014'; 
        /* JCB */
        } else if ( preg_match( "/^3\d{15}$/", $cardnum ) || preg_match("/^(2131|1800)\d{11}$/", $cardnum)) {
            $type = '007'; 
        /* Maestro */
        } else if ( preg_match( "/^(?:5020|6\\d{3})\\d{12}$/", $cardnum ) ) {
            $type = '024';     
        /* Visa Electron */
        } else if ( preg_match( "/^4(17500|917[0-9][0-9]|913[0-9][0-9]|508[0-9][0-9]|844[0-9][0-9])\d{10}$/", $cardnum ) ) {
            $type = '033';
        /*Laser*/
        } else if ( preg_match( "/^(6304|670[69]|6771)[0-9]{12,15}$/", $cardnum ) ) {
            $type = '035';
        /* Carte Blanche */
        } else if(preg_match("/^389[0-9]{11}$/", $cardnum)){
            $type = '006';
        /* Dankort */
        } else if(preg_match("/^5019\d{12}$/", $cardnum)){
            $type = '034';
        } else{
            $type = '';
        } 
        
        return $type; 
    } 

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPaymentAfter($payload)
    {
        $app = $this->application;
        $smarty = $this->smarty;

        $reasonCodes = array(
			'100' => 'Card transaction was processed successfully.',
			'102' => 'One or more fields in the request contain invalid data. <p style=\"color: #FF0000;\"><strong>Possible Action: see the reply fields invalid_fields for which fields are invalid. Resend the request with the correct information.</strong></p>',
			'104' => 'The access_key and transaction_uuid fields for this authorization request matches the access_key and transaction_uuid of another authorization request that you sent within the past 15 minutes. <p style=\"color: #FF0000;\"><strong>Possible Action: resend the request with a unique access_key and transaction_uuid fields.</strong></p>',
			'110' => 'Only a partial amount was approved.',
			'150' => 'Error - General system failure. <p style=\"color: #FF0000;\"><strong>Possible Action: See the documentation for your CyberSource client (SDK) for information about how to handle retries in the case of system errors.</strong></p>',
			'200' => 'The authorization request was approved by the issuing bank but declined by CyberSource because it did not pass Address Verification System (AVS) check. <p style=\"color: #FF0000;\"><strong>Possible Action: you can capture the authorization, but consider reviewing the order for the possibility of fraud.</strong></p>',
			'201' => 'The issuing bank has questions about the request. You do not receive an authorization code programmatically, but you might receive one verbally by calling the processor. <p style=\"color: #FF0000;\"><strong>Possible Action: call your processor to possibly receive a verbal authorization. For contact phone numbers, refer to Barclays Bank, Card Centre.</strong></p>',
			'202' => 'Expired card. You might also receive this value if the expiration date you provided does not match the date the issuing bank has on file. <p style=\"color: #FF0000;\"><strong>Possible Action: request a different card or other form of payment.</strong></p>',
			'203' => 'General decline of the card. No other information was provided by the issuing bank. <p style=\"color: #FF0000;\"><strong>Possible Action: request a different card or other form of payment.</strong></p>',
			'204' => 'Insufficient funds in the account. <p style=\"color: #FF0000;\"><strong>Possible Action: request a different card or other form of payment.</strong></p>',
			'205' => 'Stolen or lost card. <p style=\"color: #FF0000;\"><strong>Possible Action: review this transaction manually to ensure that you submitted the correct information.</strong></p>',
			'207' => 'Issuing bank unavailable. <p style=\"color: #FF0000;\"><strong>Possible Action: wait a few minutes and resend the request.</strong></p>',
			'208' => 'Inactive card or card not authorized for card-not-present transactions. <p style=\"color: #FF0000;\"><strong>Possible Action: request a different card or other form of payment.</strong></p>',
			'210' => 'The card has reached the credit limit. <p style=\"color: #FF0000;\"><strong>Possible Action: request a different card or other form of payment.</strong></p>',
			'211' => 'Invalid CVN. <p style=\"color: #FF0000;\"><strong>Possible Action: request a different card or other form of payment.</strong></p>',
			'221' => 'The customer matched an entry on the processor\'s negative file.',
			'222' => 'Account frozen or closed.',
			'230' => 'The authorization request was approved by the issuing bank but declined by CyberSource because it did not pass the Card Verification Number (CVN) check. <p style=\"color: #FF0000;\"><strong>Possible Action: you can capture the authorization, but consider reviewing the order for the possibility of fraud.</strong></p>',
			'231' => 'Invalid account number. <p style=\"color: #FF0000;\"><strong>Possible Action: request a different card or other form of payment.</strong></p>',
			'232' => 'The card type is not accepted by the payment processor. <p style=\"color: #FF0000;\"><strong>Possible Action: contact Barclays Bank, Card Centre to confirm that your account is set up to receive the card in question.</strong></p>',
			'233' => 'General decline by processor. <p style=\"color: #FF0000;\"><strong>Possible Action: request a different card or other form of payment.</strong></p>',
			'234' => 'There is a problem with the information in your CyberSource account. <p style=\"color: #FF0000;\"><strong>Possible Action: do not resend the request. Contact Barclays Bank, Card Centre to correct the information in your account.</strong></p>',
			'236' => 'Processor failure. <p style=\"color: #FF0000;\"><strong>Possible Action: wait a few minutes and resend the request.</strong></p>',
			'240' => 'The card type is invalid or does not correlate with the credit card number. <p style=\"color: #FF0000;\"><strong>Possible Action: Possible Action: confirm that the card type correlates with the credit card number specified in the request, then resend the request.</strong></p>',
			'475' => 'The cardholder is enrolled for payer authentication. <p style=\"color: #FF0000;\"><strong>Possible Action: authenticate cardholder before proceeding.</strong></p>',
			'476' => 'Payer authentication could not be authenticated.',
			'481' => 'The order has been rejected by Decision Manager.',
			'520' => 'The authorization request was approved by the issuing bank but declined by CyberSource based on your legacy Smart authorization settings. <p style=\"color: #FF0000;\"><strong>Possible Action: review the authorization request.</strong></p>',
		);

        // the api url is shared with the SOP echeck plugin, so make sure this is a card transaction before going any further
		if ( ! isset( $payload['req_payment_method'] ) || 'card' != $payload['req_payment_method'] ) return;

        // Verify the Signature before processing any further. This will ensure that no fields have been tampered with.
		if (strcmp($payload["signature"], $this->makeSignature($payload))==0) {
            $signature = ArrayUtils::get($payload, 'signature');
            $req_reference_number = ArrayUtils::get($payload, 'req_reference_number');
            $req_reference_number = explode('_', $req_reference_number);
            
            $handle = $req_reference_number[0];
            $nodeId = (int) $req_reference_number[1];
            /**
			 * Handle the transaction response according to
			 * the Decision that has been sent.
			 */
			$decision = ArrayUtils::get($payload, 'decision');
            switch ($decision) {
                case 'ACCEPT':
                    $payload['paymentId'] = $signature;
                    PaymentModule::processPayment( $nodeId, $handle, $payload );
                    $smarty->assign(array(
                        'order_url' => $app->base_uri . '/dashboard',
                        'status' => 'success'
                    ));
                    break;

                case "DECLINE":
                case "REVIEW":
                case "ERROR":
                case "CANCEL":
                    $message = ArrayUtils::get($payload, 'message');
                    $smarty->assign(array(
                        'error_message' => $message,
                        'status' => 'error'
                    ));
                    break;
                    
                default:
                    $smarty->assign(array(
                        'status' => 'error'
                    ));
                    break;
            }
        } else {
            $smarty->assign(array(
                'status' => 'error'
            ));
        }
    }

}