<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Currency;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Order\Order;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;

class Jengacheckout extends PaymentModule
{
	public $endpoint;
	public $actionUrl;
    public $accessToken;
    
    /**
    * JengaCheckout configuration
    */
    const JENGA_ENABLED = 'JENGA_ENABLED';
    const JENGA_MODE = 'JENGA_MODE';
    const JENGA_USERNAME = 'JENGA_USERNAME';
    const JENGA_PASSWORD = 'JENGA_PASSWORD';
    const JENGA_API_KEY = 'JENGA_API_KEY';
    const JENGA_OUTLET_CODE = 'JENGA_OUTLET_CODE';
    const JENGA_MERCHANT_CODE = 'JENGA_MERCHANT_CODE';

    public function __construct()
    {
        $this->name = 'jengacheckout';
        $this->icon = 'fab fa-credit-card';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Jenga Checkout';
        $this->description = 'Build a payment experience that adapts to your customers';

        $this->bootstrap = true;

        if(Configuration::get(self::JENGA_MODE) == 'live' ) {
            $this->endpoint = 'https://uat.jengahq.io';
            $this->actionUrl = 'https://test-pay.jengahq.io/cardPay';
        } else {
            $this->endpoint = 'https://sandbox.jengahq.io';
            $this->actionUrl = 'https://test-pay.jengahq.io/cardPay';
        }

        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::JENGA_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function getCountries() {
        $countries = array(
            "AU" => "Australia",
            "CA" => "Canada",
            "CH" => "China",
            "DE" => "Germany",
            "JP" => "Japan",
            "PH" => "Philippines",
            "ZA" => "South Africa",
            "AE" => "United Arab Emirates",
            "GB" => "United Kingdom",
            "US" => "United States",
            "KE" => "Kenya"
        );

        return $countries;
    }

    public function getAccessToken() {
        // Payload Details
        $payload = array(
            'username' 	=> Configuration::get(self::JENGA_USERNAME),
            'password' 	=> Configuration::get(self::JENGA_PASSWORD)
        );

        if(Configuration::get(self::JENGA_MODE) == 'live' ) {
            $endpoint = $this->endpoint . '/identity/v2/token';
        } else {
            $endpoint = $this->endpoint . '/identity-test/v2/token';
        }

        $ch = curl_init( $endpoint );
        curl_setopt($ch, CURLOPT_URL, $endpoint );
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: " . Configuration::get(self::JENGA_API_KEY),
            "Content-Type: application/x-www-form-urlencoded",
        ));
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($payload));
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);
        $response = Tools::jsonDecode($response, true);

        //  return $response;
        if ( !ArrayUtils::get($response, 'access_token') ) {
            throw new Exception('Jenga PGW Response did not get any data.');
        }

        $accessToken = ArrayUtils::get($response, 'access_token');
        $this->accessToken = $accessToken;

        return $accessToken;
    }

    public function createBill($customerName, $refrenceId, $billerCode, $amount, $currency) {
        // Payload Details
        $payload = array(
            'customer'	=>array(
                'name' => $customerName
            ),
            'order' => array(
                "reference" => $refrenceId,
                "amount" => $amount,
                "currency" => $currency,
                "channel" => "CARD",
                "description" => "Payment of " . formatPrice($amount),
                "outlet" => Configuration::get(self::JENGA_OUTLET_CODE),
                "billerCode" => $billerCode
            )
        );

        $endpoint = $this->endpoint . '/payment/v2/bills';

        $ch = curl_init( $endpoint );
        curl_setopt($ch, CURLOPT_URL, $endpoint );
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: Bearer " . $this->accessToken,
            "Content-Type: application/json",
        ));
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($payload));
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);
        $response = Tools::jsonDecode($response, true);

        if ( ArrayUtils::get($response, 'status') != '1' ) {
            throw new Exception('Jenga PGW Response did not get any data.');
        }

        return $response;
    }

    public function processOrderCheckout(Order $order, Currency $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $accessToken = $this->getAccessToken();

        $billerCode = Tools::randomGen(10);

        $bill = $this->createBill(
            $user->name, 
            $order->id, 
            $user->id, 
            $order->total, 
            $currency->iso_code
        );

        if(is_array($bill) && ArrayUtils::has($bill, 'reference')) {
            $billReference = ArrayUtils::get($bill, 'reference');

            $smarty->assign([
                'countries' => $this->getCountries(),
                'css_path' => $this->local_path . 'views/_css.tpl',
                'base_uri' => $app->base_uri,
                'redirectUri' => $app->base_uri . '/dashboard/order/' . $order->id,
                'actionUrl' => $this->actionUrl,
                'accessToken' => $accessToken,
                'outletCode' => Configuration::get(self::JENGA_OUTLET_CODE),
                'merchantCode' => Configuration::get(self::JENGA_MERCHANT_CODE),
                'billReference' => $billReference,
                'payee_email' => $user->email,
                'serviceDate' => DateUtils::now(),
                'ipAddress' => $app->request->getIp()
            ]);
            
            return $this->getTemplateContent('jenga.order');
        }
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPaymentAfter($payload) 
    { 
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $handle = ArrayUtils::get($payload, 'handle');
        $nodeId = ArrayUtils::get($payload, 'nodeId');

        $accessToken = $this->accessToken;
        $endpoint = $this->endpoint . '/payment/v2/query?billReference=' . $nodeId;

        $ch = curl_init( $endpoint );
        curl_setopt($ch, CURLOPT_URL, $endpoint);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: Bearer " . $accessToken,
            "Content-Type: application/json",
        ));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        $response = curl_exec($ch);
        $transaction = Tools::jsonDecode($response, true);

        if ( !empty(ArrayUtils::get($transaction, 'transactionRef')) ) {
            $transactionRef = ArrayUtils::get($transaction, 'transactionRef');
            $orderAmount = ArrayUtils::get($transaction, 'orderAmount');
            $orderReference = ArrayUtils::get($transaction, 'orderReference');

            $params['paymentId'] = $transactionRef;
            $params['amount'] = $orderAmount;
            $params['customer_id'] = $user->id;
            PaymentModule::processPayment( $orderReference, $handle, $payload );
            $smarty->assign(array(
                'status' => 'success'
            ));
        } else {
            $smarty->assign(array(
                'status' => 'error'
            ));
        }

        print_r($payload);
        die();

    }
    
}