<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Site\Site;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

class PayPal extends PaymentModule
{
    /**
    * PayPal configuration
    */
    const PAYPAL_ENABLED = 'PAYPAL_ENABLED';
    const PAYPAL_MODE = 'PAYPAL_MODE';
    const PAYPAL_CLIENT_ID = 'PAYPAL_CLIENT_ID';
    const PAYPAL_SECRET_KEY = 'PAYPAL_SECRET_KEY';
    const PAYPAL_INTEGRATION_MODE = 'PAYPAL_INTEGRATION_MODE';
    const PAYPAL_EMAIL = 'PAYPAL_EMAIL';

    /** @var string paypalUrl */
    public $paypalUrl;

    public function __construct()
    {
        $this->name = 'paypal';
        $this->icon = 'fab fa-paypal';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'PayPal';
        $this->description = 'Accepts payments by PayPal';

        $enableSandbox = Configuration::get( self::PAYPAL_MODE ) == 'sandbox' ? true : false;
        $this->paypalUrl = $enableSandbox ? 'https://www.sandbox.paypal.com/cgi-bin/webscr' : 'https://www.paypal.com/cgi-bin/webscr';
    }

    public function isActive() {
        return Configuration::get(self::PAYPAL_ENABLED);
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function verifyTransaction($data) {
        $req = 'cmd=_notify-validate';
        foreach ($data as $key => $value) {
            $value = urlencode(stripslashes($value));
            $value = preg_replace('/(.*[^%^0^D])(%0A)(.*)/i', '${1}%0D%0A${3}', $value); // IPN fix
            $req .= "&$key=$value";
        }
    
        $ch = curl_init($this->paypalUrl);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $req);
        curl_setopt($ch, CURLOPT_SSLVERSION, 6);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_FORBID_REUSE, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Connection: Close'));
        $res = curl_exec($ch);
    
        if (!$res) {
            $errno = curl_errno($ch);
            $errstr = curl_error($ch);
            curl_close($ch);
            throw new Exception("cURL error: [$errno] $errstr");
        }
    
        $info = curl_getinfo($ch);
    
        // Check the http response
        $httpCode = $info['http_code'];
        if ($httpCode != 200) {
            throw new Exception("PayPal responded with http code $httpCode");
        }
        curl_close($ch);
    
        return $res === 'VERIFIED';
    }

    public function checkTxnid($txnid) {
        $paymentExists = (bool) Db::getInstance()->getValue('SELECT * FROM ' . Db::prefix('payment') . ' WHERE transaction_id =  \'' . pSQL($txnid) . '\'');
        return $paymentExists;
    }

    public function processOrderCheckout($order, $currency, $checkoutOptions = array()) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        if(Configuration::get(self::PAYPAL_INTEGRATION_MODE) == 'paypal_email') {
            $paypalConfig = [
                'email' => Configuration::get( self::PAYPAL_EMAIL ),
                'return_url' => ArrayUtils::get($checkoutOptions, 'success_url'),
                'cancel_url' => ArrayUtils::get($checkoutOptions, 'decline_url'),
                'notify_url' => ArrayUtils::get($checkoutOptions, 'callback_url')
            ];
 
            $data = array(
                "cmd" => "_xclick",
                "no_note" => "1",
                "lc" => "US",
                "bn" => "PP-BuyNowBF:btn_buynow_LG.gif:NonHostedGuest",
                "payer_email" => $user->email,
                "item_number" => $order->id,
                'business' => Configuration::get( self::PAYPAL_EMAIL ),
            );

            $data['return'] = stripslashes($paypalConfig['return_url']);
            $data['cancel_return'] = stripslashes($paypalConfig['cancel_url']);
            $data['notify_url'] = stripslashes($paypalConfig['notify_url']);

            // Set the details about the product being purchased, including the amount
            // and currency so that these aren't overridden by the form data.
            $options['currency'] = array(
                'symbol' => $currency->symbol,
                'rate' => $currency->conversion_rate
            );

            $data['item_name'] = ArrayUtils::get($checkoutOptions, 'title');
            $data['amount'] = ArrayUtils::get($checkoutOptions, 'amount');
            $data['currency_code'] = ArrayUtils::get($checkoutOptions, 'currency');

            // Build the query string from the data.
            $queryString = http_build_query($data);

            // Redirect to paypal IPN
            header('location:' . $this->paypalUrl . '?' . $queryString);
            exit();
        }

        $smarty->assign([
            'paypal_mode' => Configuration::get( self::PAYPAL_MODE ),
            'paypal_client' => Configuration::get( self::PAYPAL_CLIENT_ID ),
            'paypal_secret' => Configuration::get( self::PAYPAL_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paypal.order');
    }

    public function processPaperCheckout($paper, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'paypal_mode' => Configuration::get( self::PAYPAL_MODE ),
            'paypal_client' => Configuration::get( self::PAYPAL_CLIENT_ID ),
            'paypal_secret' => Configuration::get( self::PAYPAL_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paypal.paper');
    }

    public function processAddPaymentCheckout($addPayment, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'paypal_mode' => Configuration::get( self::PAYPAL_MODE ),
            'paypal_client' => Configuration::get( self::PAYPAL_CLIENT_ID ),
            'paypal_secret' => Configuration::get( self::PAYPAL_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paypal.add-payment');
    }

    public function processWalletCheckout($user_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'paypal_mode' => Configuration::get( self::PAYPAL_MODE ),
            'paypal_client' => Configuration::get( self::PAYPAL_CLIENT_ID ),
            'paypal_secret' => Configuration::get( self::PAYPAL_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paypal.wallet');
    }

    public function processTipCheckout($order_id, $amount, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        $smarty->assign([
            'paypal_mode' => Configuration::get( self::PAYPAL_MODE ),
            'paypal_client' => Configuration::get( self::PAYPAL_CLIENT_ID ),
            'paypal_secret' => Configuration::get( self::PAYPAL_SECRET_KEY ),
            'payee_email' => $user->email
        ]);
        
        return $this->getTemplateContent('paypal.tip');
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */ 
    public function verifyPayment($node_id, $handle, $payload) 
    { 
        $payment_method = ArrayUtils::get($payload, 'payment_method');

        if (ArrayUtils::has($payload, 'txn_id') && ArrayUtils::has($payload, 'txn_type')) {
            unset($payload['payment_method']);
            if ($this->verifyTransaction($payload) && $this->checkTxnid(ArrayUtils::get($payload, 'txn_id'))) {
                $payload['payment_method'] = $payment_method;
                $payload['paymentId'] = ArrayUtils::get($payload, 'txn_id');
                $payload['amount'] = (float) ArrayUtils::get($payload, 'mc_gross');
                $node_id = (int) ArrayUtils::get($payload, 'item_number');
                $payload['nodeId'] = $node_id;

                PaymentModule::processPayment( $node_id, $handle, $payload );
            }
        } else {
            PaymentModule::processPayment( $node_id, $handle, $payload );
        }
    }

}