<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Module\PaymentModule;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

require_once dirname(__FILE__) . '/vendor/autoload.php';

class Two_Checkout extends PaymentModule
{
    /**
    * 2Checkout configuration
    */
    const TWO_CHECKOUT_ENABLED = 'TWO_CHECKOUT_ENABLED';
    const TWO_CHECKOUT_MODE = 'TWO_CHECKOUT_MODE';
    const TWO_CHECKOUT_MERCHANT_CODE = 'TWO_CHECKOUT_MERCHANT_CODE';
    const TWO_CHECKOUT_API_PUBLISHABLE_KEY = 'TWO_CHECKOUT_API_PUBLISHABLE_KEY';
    const TWO_CHECKOUT_API_PRIVATE_KEY = 'TWO_CHECKOUT_API_PRIVATE_KEY';

    public function __construct()
    {
        $this->name = 'two_checkout';
        $this->icon = 'fab fa-credit-card';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = '2Checkout';
        $this->description = '2Checkout allows you to accept payment on your site via credit card';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function isActive() {
        return Configuration::get(self::TWO_CHECKOUT_ENABLED);
    }

    public function getCountries() {
        $countries = array(
            "AU" => "Australia",
            "CA" => "Canada",
            "CH" => "China",
            "DE" => "Germany",
            "JP" => "Japan",
            "PH" => "Philippines",
            "ZA" => "South Africa",
            "AE" => "United Arab Emirates",
            "GB" => "United Kingdom",
            "US" => "United States",
            "KE" => "Kenya"
        );

        return $countries;
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function processOrderCheckout($order, $currency) {
        $app = $this->application;
        $smarty = $this->smarty;
        $cookie = $app->container->cookie;
        $user = $app->container->user;

        $marchant_code = Configuration::get(self::TWO_CHECKOUT_MERCHANT_CODE);
        $api_publishable_key = Configuration::get(self::TWO_CHECKOUT_API_PUBLISHABLE_KEY);
        $checkout_mode = Configuration::get(self::TWO_CHECKOUT_MODE);

        $smarty->assign([
            'base_uri' => $app->base_uri,
            'marchant_code' => $marchant_code,
            'api_publishable_key' => $api_publishable_key,
            'checkout_mode' => $checkout_mode,
            'js_path' => $this->_url_path . 'js/2checkout.js',
            'css_path' => $this->local_path . 'views/_css.2checkout.tpl',
            'form_path' => $this->local_path . 'views/2checkout.form.tpl',
            'countries' => $this->getCountries(),
            'redirectUri' => $app->base_uri . '/dashboard/order/' . $order->id
        ]);

        return $this->getTemplateContent('2checkout.order');
    }

    /**
     * return_json
     * 
     * @param array $response
     * @return json
     */
    public function returnJson($response = array()) {
        header('Content-Type: application/json');
        exit(json_encode($response));
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPayment($node_id, $handle, $payload) 
    { 
        $app = $this->application;
        $smarty = $this->smarty;

        $token = ArrayUtils::get($payload, 'token');
        $amount = ArrayUtils::get($payload, 'amount');
        $currency = ArrayUtils::get($payload, 'currency');
        $redirect_uri = ArrayUtils::get($payload, 'redirect_uri');
        $return_url = ArrayUtils::get($payload, 'return_url');

        $billing_first_name = ArrayUtils::get($payload, 'billing_first_name');
        $billing_last_name = ArrayUtils::get($payload, 'billing_last_name');
        $billing_address = ArrayUtils::get($payload, 'billing_address');
        $billing_email = ArrayUtils::get($payload, 'billing_email');
        $billing_phone = ArrayUtils::get($payload, 'billing_phone');
        $billing_state = ArrayUtils::get($payload, 'billing_state');
        $bill_to_address_city = ArrayUtils::get($payload, 'bill_to_address_city');
        $billing_zip_code = ArrayUtils::get($payload, 'billing_zip_code');
        $billing_country = ArrayUtils::get($payload, 'billing_country');

        $marchant_code = Configuration::get(self::TWO_CHECKOUT_MERCHANT_CODE);
        $api_private_key = Configuration::get(self::TWO_CHECKOUT_API_PRIVATE_KEY);
        $checkout_mode = Configuration::get(self::TWO_CHECKOUT_MODE);

        Twocheckout::privateKey( $api_private_key );
        Twocheckout::sellerId( $marchant_code );
        if($checkout_mode == 'sandbox') {
            Twocheckout::verifySSL(false);
        }

        $twocheckout_config = array(
            "sellerId" => $marchant_code,
            "merchantOrderId" => $node_id,
            "token" => $token,
            "currency" => $currency,
            "total" => $amount,
            "billingAddr" => array(
                "firstName" => $billing_first_name,
                "lastName" => $billing_last_name,
                "addrLine1" => $billing_address,
                "city" => $bill_to_address_city,
                "state" => $billing_state,
                "zipCode" => $billing_zip_code,
                "country" => $billing_country,
                "email" => $billing_email,
                "phoneNumber" => $billing_phone
            ),
            "returnUrl" => $return_url
        );

        if($checkout_mode == 'sandbox') {
            $twocheckout_config['demo'] = 'Y';
        }

        try {
            $charge = Twocheckout_Charge::auth($twocheckout_config);
            // successful
            if (isset($charge['response']['responseCode']) && $charge['response']['responseCode'] == 'APPROVED') {
                $transactionId = $charge['response']['transactionId'];
                $payload['paymentId'] = $transactionId;
                PaymentModule::processPayment( $node_id, $handle, $payload );
                $redirect_uri = $app->base_uri.'/payment/two_checkout/success?redirectUri=' . $redirect_uri;
                $app->response([
                    'callback' => 'window.location.href = "'.$redirect_uri.'";' 
                ]);
            // 3DS
            } elseif( isset($charge['exception']['errorCode']) && $charge['exception']['errorCode'] == '700') {
                $app->response([
                    'callback' => 'window.location.href = "'.$charge['exception']['errorMsg'].'";' 
                ]);
            } else {
                $app->response([
                    'error' => true,
                    'message' => 'Payment Declined: Please verify your information and try again, or try another payment method'
                ]);
            }
        } catch (Exception $e) {
            $app->response([
                'error' => true,
                'message' => 'Payment Declined: ' . $e->getMessage()
            ]);
        }
    }

    /**
    * Hook action verify payment
    * @param $params array
    * @return void
    */
    public function verifyPaymentAfter($params) 
    { 
        $app = $this->application;
        $smarty = $this->smarty;
        $user = $app->container->user;

        if(ArrayUtils::has($params, 'transactionId')) {
            $handle = ArrayUtils::get($params, 'handle');
            $nodeId = (int) ArrayUtils::get($params, 'merchantOrderId');
            $transactionId = ArrayUtils::get($params, 'transactionId');
            $responseCode = ArrayUtils::get($params, 'responseCode');
            PaymentModule::processPayment( $nodeId, $handle, $params );
        }
    }
}